=== modified file 'loggerhead/branchview.py'
--- loggerhead/branchview.py	2007-03-06 04:27:05 +0000
+++ loggerhead/branchview.py	2007-08-23 15:03:26 +0000
@@ -29,7 +29,7 @@
 from cherrypy import HTTPRedirect
 
 from loggerhead import util
-from loggerhead.changecache import ChangeCache
+from loggerhead.changecache import ChangeCache, FileChangeCache
 from loggerhead.history import History
 from loggerhead.textindex import TextIndex
 from loggerhead.controllers.changelog_ui import ChangeLogUI
@@ -141,14 +141,16 @@
             self.log.debug('Reload branch history...')
             if self._history is not None:
                 self._history.detach()
-            self._history = History.from_folder(self._absfolder, self._name)
+            _history = self._history = History.from_folder(
+                self._absfolder, self._name)
             cache_path = self._config.get('cachepath', None)
             if cache_path is None:
                 # try the project config
                 cache_path = self._project_config.get('cachepath', None)
             if cache_path is not None:
-                self._history.use_cache(ChangeCache(self._history, cache_path))
-                self._history.use_search_index(TextIndex(self._history, cache_path))
+                _history.use_cache(ChangeCache(_history, cache_path))
+                _history.use_file_cache(FileChangeCache(_history, cache_path))
+                _history.use_search_index(TextIndex(_history, cache_path))
         return self._history
     
     def check_rebuild(self):

=== modified file 'loggerhead/changecache.py'
--- loggerhead/changecache.py	2007-06-04 21:16:03 +0000
+++ loggerhead/changecache.py	2007-09-07 16:41:32 +0000
@@ -25,9 +25,9 @@
 cached a change, it's good forever.
 """
 
+import cPickle
 import logging
 import os
-import shelve
 import threading
 import time
 
@@ -37,10 +37,73 @@
 
 
 with_lock = util.with_lock('_lock', 'ChangeCache')
-        
+
+SQLITE_INTERFACE = os.environ.get('SQLITE_INTERFACE', 'sqlite')
+
+if SQLITE_INTERFACE == 'pysqlite2':
+    from pysqlite2 import dbapi2
+    _param_marker = '?'
+elif SQLITE_INTERFACE == 'sqlite':
+    import sqlite as dbapi2
+    _param_marker = '%s'
+else:
+    raise AssertionError("bad sqlite interface %r!?"%SQLITE_INTERFACE)
+
+_select_stmt = ("select data from revisiondata where revid = ?"
+                ).replace('?', _param_marker)
+_insert_stmt = ("insert into revisiondata (revid, data) "
+                "values (?, ?)").replace('?', _param_marker)
+_update_stmt = ("update revisiondata set data = ? where revid = ?"
+                ).replace('?', _param_marker)
+
+
+
+
+class FakeShelf(object):
+    def __init__(self, filename):
+        create_table = not os.path.exists(filename)
+        self.connection = dbapi2.connect(filename)
+        self.cursor = self.connection.cursor()
+        if create_table:
+            self._create_table()
+    def _create_table(self):
+        self.cursor.execute(
+            "create table RevisionData "
+            "(revid binary primary key, data binary)")
+        self.connection.commit()
+    def _serialize(self, obj):
+        r = dbapi2.Binary(cPickle.dumps(obj, protocol=2))
+        return r
+    def _unserialize(self, data):
+        return cPickle.loads(str(data))
+    def get(self, revid):
+        self.cursor.execute(_select_stmt, (revid,))
+        filechange = self.cursor.fetchone()
+        if filechange is None:
+            return None
+        else:
+            return self._unserialize(filechange[0])
+    def add(self, revid_obj_pairs, commit=True):
+        for  (r, d) in revid_obj_pairs:
+            self.cursor.execute(_insert_stmt, (r, self._serialize(d)))
+        if commit:
+            self.connection.commit()
+    def update(self, revid_obj_pairs, commit=True):
+        for  (r, d) in revid_obj_pairs:
+            self.cursor.execute(_update_stmt, (self._serialize(d), r))
+        if commit:
+            self.connection.commit()
+    def count(self):
+        self.cursor.execute(
+            "select count(*) from revisiondata")
+        return self.cursor.fetchone()[0]
+    def close(self, commit=False):
+        if commit:
+            self.connection.commit()
+        self.connection.close()
 
 class ChangeCache (object):
-    
+
     def __init__(self, history, cache_path):
         self.history = history
         self.log = history.log
@@ -48,20 +111,21 @@
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
 
-        # keep a separate cache for the diffs, because they're very time-consuming to fetch.
-        self._changes_filename = os.path.join(cache_path, 'changes')
-        self._changes_diffs_filename = os.path.join(cache_path, 'changes-diffs')
-        
+        self._changes_filename = os.path.join(cache_path, 'changes.sql')
+
         # use a lockfile since the cache folder could be shared across different processes.
         self._lock = LockFile(os.path.join(cache_path, 'lock'))
         self._closed = False
-        
-        # this is fluff; don't slow down startup time with it.
-        def log_sizes():
-            s1, s2 = self.sizes()
-            self.log.info('Using change cache %s; %d/%d entries.' % (cache_path, s1, s2))
-        threading.Thread(target=log_sizes).start()
-    
+
+##         # this is fluff; don't slow down startup time with it.
+##         # but it is racy in tests :(
+##         def log_sizes():
+##             self.log.info('Using change cache %s; %d entries.' % (cache_path, self.size()))
+##         threading.Thread(target=log_sizes).start()
+
+    def _cache(self):
+        return FakeShelf(self._changes_filename)
+
     @with_lock
     def close(self):
         self.log.debug('Closing cache file.')
@@ -76,66 +140,49 @@
         pass
     
     @with_lock
-    def get_changes(self, revid_list, get_diffs=False):
+    def get_changes(self, revid_list):
         """
         get a list of changes by their revision_ids.  any changes missing
         from the cache are fetched by calling L{History.get_change_uncached}
         and inserted into the cache before returning.
         """
-        if get_diffs:
-            cache = shelve.open(self._changes_diffs_filename, 'c', protocol=2)
-        else:
-            cache = shelve.open(self._changes_filename, 'c', protocol=2)
-
-        try:
-            out = []
-            fetch_list = []
-            sfetch_list = []
-            for revid in revid_list:
-                # if the revid is in unicode, use the utf-8 encoding as the key
-                srevid = util.to_utf8(revid)
-                
-                if srevid in cache:
-                    out.append(cache[srevid])
-                else:
-                    #self.log.debug('Entry cache miss: %r' % (revid,))
-                    out.append(None)
-                    fetch_list.append(revid)
-                    sfetch_list.append(srevid)
-            
-            if len(fetch_list) > 0:
-                # some revisions weren't in the cache; fetch them
-                changes = self.history.get_changes_uncached(fetch_list, get_diffs)
-                if changes is None:
-                    return changes
-                for i in xrange(len(revid_list)):
-                    if out[i] is None:
-                        cache[sfetch_list.pop(0)] = out[i] = changes.pop(0)
-            return out
-        finally:
-            cache.close()
-    
-    @with_lock
-    def full(self, get_diffs=False):
-        if get_diffs:
-            cache = shelve.open(self._changes_diffs_filename, 'c', protocol=2)
-        else:
-            cache = shelve.open(self._changes_filename, 'c', protocol=2)
-        try:
-            return (len(cache) >= len(self.history.get_revision_history())) and (util.to_utf8(self.history.last_revid) in cache)
-        finally:
-            cache.close()
-
-    @with_lock
-    def sizes(self):
-        cache = shelve.open(self._changes_filename, 'c', protocol=2)
-        s1 = len(cache)
-        cache.close()
-        cache = shelve.open(self._changes_diffs_filename, 'c', protocol=2)
-        s2 = len(cache)
-        cache.close()
-        return s1, s2
-        
+        out = []
+        missing_revids = []
+        missing_revid_indices = []
+        cache = self._cache()
+        for revid in revid_list:
+            entry = cache.get(revid)
+            if entry is not None:
+                out.append(entry)
+            else:
+                missing_revids.append(revid)
+                missing_revid_indices.append(len(out))
+                out.append(None)
+        if missing_revids:
+            missing_entries = self.history.get_changes_uncached(missing_revids)
+            missing_entry_dict = {}
+            for entry in missing_entries:
+                missing_entry_dict[entry.revid] = entry
+            revid_entry_pairs = []
+            for i, revid in zip(missing_revid_indices, missing_revids):
+                out[i] = entry = missing_entry_dict.get(revid)
+                if entry is not None:
+                    revid_entry_pairs.append((revid, entry))
+            cache.add(revid_entry_pairs)
+        return filter(None, out)
+
+    @with_lock
+    def full(self):
+        cache = self._cache()
+        last_revid = util.to_utf8(self.history.last_revid)
+        revision_history = self.history.get_revision_history()
+        return (cache.count() >= len(revision_history)
+                and cache.get(last_revid) is not None)
+
+    @with_lock
+    def size(self):
+        return self._cache().count()
+
     def check_rebuild(self, max_time=3600):
         """
         check if we need to fill in any missing pieces of the cache.  pull in
@@ -174,4 +221,39 @@
         self.log.info('Revision cache rebuild completed.')
         self.flush()
 
-
+class FileChangeCache(object):
+    def __init__(self, history, cache_path):
+        self.history = history
+
+        if not os.path.exists(cache_path):
+            os.mkdir(cache_path)
+
+        self._changes_filename = os.path.join(cache_path, 'filechanges.sql')
+
+        # use a lockfile since the cache folder could be shared across
+        # different processes.
+        self._lock = LockFile(os.path.join(cache_path, 'filechange-lock'))
+
+    @with_lock
+    def get_file_changes(self, entries):
+        out = []
+        missing_entries = []
+        missing_entry_indices = []
+        cache = FakeShelf(self._changes_filename)
+        for entry in entries:
+            changes = cache.get(entry.revid)
+            if changes is not None:
+                out.append(changes)
+            else:
+                missing_entries.append(entry)
+                missing_entry_indices.append(len(out))
+                out.append(None)
+        if missing_entries:
+            missing_changes = self.history.get_file_changes_uncached(missing_entries)
+            revid_changes_pairs = []
+            for i, entry, changes in zip(
+                missing_entry_indices, missing_entries, missing_changes):
+                revid_changes_pairs.append((entry.revid, changes))
+                out[i] = changes
+            cache.add(revid_changes_pairs)
+        return out

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2007-06-18 23:07:53 +0000
+++ loggerhead/controllers/__init__.py	2007-06-19 00:27:53 +0000
@@ -61,7 +61,6 @@
             self._recheck_auto_folders()
     
     def _recheck_auto_folders(self):
-        log.debug('recheck...')
         if self._auto_folder is None:
             return
         auto_list = []
@@ -74,7 +73,6 @@
         auto_list.sort()
         if auto_list == self._auto_list:
             # nothing has changed; do nothing.
-            log.debug('ok')
             return
 
         # rebuild views:
@@ -109,12 +107,11 @@
     def index(self):
         for p in self._projects:
             p._recheck_auto_folders()
-        log.debug('%r' % {
+        return {
             'projects': self._projects,
             'util': util,
             'title': self._config.get('title', ''),
-        })
-        return {}
+        }
 
     def _check_rebuild(self):
         for p in self._projects:

=== modified file 'loggerhead/controllers/annotate_ui.py'
--- loggerhead/controllers/annotate_ui.py	2007-05-21 07:37:15 +0000
+++ loggerhead/controllers/annotate_ui.py	2007-08-23 12:18:18 +0000
@@ -52,49 +52,48 @@
         z = time.time()
         h = self._branch.get_history()
         util.set_context(kw)
-        
-        if len(args) > 0:
-            revid = h.fix_revid(args[0])
-        else:
-            revid = None
-        path = None
-        if len(args) > 1:
-            path = '/'.join(args[1:])
-            if not path.startswith('/'):
-                path = '/' + path
-        
-        file_id = kw.get('file_id', None)
-        if (file_id is None) and (path is None):
-            raise HTTPError(400, 'No file_id or filename provided to annotate')
-
-        if file_id is None:
-            file_id = h.get_file_id(revid, path)
-
+
+        h._branch.lock_read()
         try:
-            revid_list, revid = h.get_file_view(revid, file_id)
-        except:
-            self.log.exception('Exception fetching changes')
-            raise InternalError('Could not fetch changes')
-            
-        # no navbar for revisions
-        navigation = util.Container()
-        
-        if path is None:
-            path = h.get_path(revid, file_id)
-        filename = os.path.basename(path)
-
-        vals = {
-            'branch': self._branch,
-            'util': util,
-            'revid': revid,
-            'file_id': file_id,
-            'path': path,
-            'filename': filename,
-            'history': h,
-            'navigation': navigation,
-            'change': h.get_changes([ revid ])[0],
-            'contents': list(h.annotate_file(file_id, revid)),
-        }
-        h.flush_cache()
-        self.log.info('/annotate: %r secs' % (time.time() - z,))
-        return vals
+            if len(args) > 0:
+                revid = h.fix_revid(args[0])
+            else:
+                revid = h.last_revid
+
+            path = None
+            if len(args) > 1:
+                path = '/'.join(args[1:])
+                if not path.startswith('/'):
+                    path = '/' + path
+
+            file_id = kw.get('file_id', None)
+            if (file_id is None) and (path is None):
+                raise HTTPError(400, 'No file_id or filename provided to annotate')
+
+            if file_id is None:
+                file_id = h.get_file_id(revid, path)
+
+            # no navbar for revisions
+            navigation = util.Container()
+
+            if path is None:
+                path = h.get_path(revid, file_id)
+            filename = os.path.basename(path)
+
+            vals = {
+                'branch': self._branch,
+                'util': util,
+                'revid': revid,
+                'file_id': file_id,
+                'path': path,
+                'filename': filename,
+                'history': h,
+                'navigation': navigation,
+                'change': h.get_changes([ revid ])[0],
+                'contents': list(h.annotate_file(file_id, revid)),
+            }
+            h.flush_cache()
+            self.log.info('/annotate: %r secs' % (time.time() - z,))
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/atom_ui.py'
--- loggerhead/controllers/atom_ui.py	2007-03-29 06:55:55 +0000
+++ loggerhead/controllers/atom_ui.py	2007-08-30 11:39:10 +0000
@@ -33,17 +33,22 @@
     @turbogears.expose(template='loggerhead.templates.atom', format="xml", content_type="application/atom+xml")
     def default(self, *args):
         h = self._branch.get_history()
-        pagesize = int(self._branch.config.get('pagesize', '20'))
-
-        revid_list, start_revid = h.get_file_view(None, None)
-        entries = list(h.get_changes(list(revid_list)[:pagesize]))
-
-        vals = {
-            'branch': self._branch,
-            'changes': entries,
-            'util': util,
-            'history': h,
-            'updated': entries[0].date.isoformat() + 'Z',
-        }
-        h.flush_cache()
-        return vals
+
+        h._branch.lock_read()
+        try:
+            pagesize = int(self._branch.config.get('pagesize', '20'))
+
+            revid_list = h.get_file_view(h.last_revid, None)
+            entries = list(h.get_changes(list(revid_list)[:pagesize]))
+
+            vals = {
+                'branch': self._branch,
+                'changes': entries,
+                'util': util,
+                'history': h,
+                'updated': entries[0].date.isoformat() + 'Z',
+            }
+            h.flush_cache()
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/bundle_ui.py'
--- loggerhead/controllers/bundle_ui.py	2007-03-14 06:10:22 +0000
+++ loggerhead/controllers/bundle_ui.py	2007-08-23 12:18:18 +0000
@@ -34,26 +34,30 @@
 
     @turbogears.expose()
     def default(self, *args, **kw):
-        # /bundle/<rev_id>/filename
+        # /bundle/<rev_id>/[<compare_rev_id>/]filename
         z = time.time()
         h = self._branch.get_history()
-        compare_revid = kw.get('compare_revid', None)
-
-        if len(args) < 1:
-            raise HTTPRedirect(self._branch.url('/changes'))
-        revid = h.fix_revid(args[0])
-        compare_revid = None
-        if len(args) >= 3:
-            compare_revid = h.fix_revid(args[1])
-
+
+        h._branch.lock_read()
         try:
-            bundle_data = h.get_bundle(revid, compare_revid)
-        except:
-            self.log.exception('Exception fetching bundle')
-            raise InternalError('Could not fetch bundle')
-            
-        response.headers['Content-Type'] = 'text/plain'
-        response.headers['Content-Length'] = len(bundle_data)
-        response.body = bundle_data
-        self.log.info('/bundle: %r seconds' % (time.time() - z,))
-        return response.body
+            if len(args) < 1:
+                raise HTTPRedirect(self._branch.url('/changes'))
+            revid = h.fix_revid(args[0])
+            if len(args) >= 3:
+                compare_revid = h.fix_revid(args[1])
+            else:
+                compare_revid = None
+
+            try:
+                bundle_data = h.get_bundle(revid, compare_revid)
+            except:
+                self.log.exception('Exception fetching bundle')
+                raise InternalError('Could not fetch bundle')
+
+            response.headers['Content-Type'] = 'text/plain'
+            response.headers['Content-Length'] = len(bundle_data)
+            response.body = bundle_data
+            self.log.info('/bundle: %r seconds' % (time.time() - z,))
+            return response.body
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/changelog_ui.py'
--- loggerhead/controllers/changelog_ui.py	2007-03-14 06:10:22 +0000
+++ loggerhead/controllers/changelog_ui.py	2007-08-23 12:18:18 +0000
@@ -39,60 +39,82 @@
         z = time.time()
         h = self._branch.get_history()
         config = self._branch.config
-        
-        if len(args) > 0:
-            revid = h.fix_revid(args[0])
-        else:
-            revid = None
 
-        file_id = kw.get('file_id', None)
-        query = kw.get('q', None)
-        start_revid = h.fix_revid(kw.get('start_revid', None))
-        orig_start_revid = start_revid
-        pagesize = int(config.get('pagesize', '20'))
-        search_failed = False
-        
+        h._branch.lock_read()
         try:
-            revid, start_revid, revid_list = h.get_view(revid, start_revid, file_id, query)
-            kw['start_revid'] = start_revid
-            util.set_context(kw)
-            
-            if (query is not None) and (len(revid_list) == 0):
-                search_failed = True
-
-            if len(revid_list) == 0:
-                scan_list = revid_list
+            if len(args) > 0:
+                revid = h.fix_revid(args[0])
             else:
-                scan_list = revid_list[h.get_revid_sequence(revid_list, revid):]
-            entry_list = scan_list[:pagesize]
-            entries = h.get_changes(entry_list)
-        except:
-            self.log.exception('Exception fetching changes')
-            raise InternalError('Could not fetch changes')
-
-        navigation = util.Container(pagesize=pagesize, revid=revid, start_revid=start_revid, revid_list=revid_list,
-                                    file_id=file_id, scan_url='/changes', branch=self._branch, feed=True)
-        if query is not None:
-            navigation.query = query
-        util.fill_in_navigation(h, navigation)
-        
-        entries = list(entries)
-        # add parent & merge-point branch-nick info, in case it's useful
-        h.get_branch_nicks(entries)
-
-        vals = {
-            'branch': self._branch,
-            'changes': list(entries),
-            'util': util,
-            'history': h,
-            'revid': revid,
-            'navigation': navigation,
-            'file_id': file_id,
-            'start_revid': start_revid,
-            'viewing_from': (orig_start_revid is not None) and (orig_start_revid != h.last_revid),
-            'query': query,
-            'search_failed': search_failed,
-        }
-        h.flush_cache()
-        self.log.info('/changes %r: %r secs' % (revid, time.time() - z))
-        return vals
+                revid = None
+
+            file_id = kw.get('file_id', None)
+            query = kw.get('q', None)
+            start_revid = h.fix_revid(kw.get('start_revid', None))
+            orig_start_revid = start_revid
+            pagesize = int(config.get('pagesize', '20'))
+            search_failed = False
+
+            try:
+                revid, start_revid, revid_list = h.get_view(revid, start_revid, file_id, query)
+                kw['start_revid'] = start_revid
+                util.set_context(kw)
+
+                if (query is not None) and (len(revid_list) == 0):
+                    search_failed = True
+
+                if len(revid_list) == 0:
+                    scan_list = revid_list
+                else:
+                    if revid in revid_list: # XXX is this always true?
+                        i = revid_list.index(revid)
+                    else:
+                        i = None
+                    scan_list = revid_list[i:]
+                entry_list = scan_list[:pagesize]
+                entries = h.get_changes(entry_list)
+                h.add_changes(entries)
+            except:
+                self.log.exception('Exception fetching changes')
+                raise InternalError('Could not fetch changes')
+
+            navigation = util.Container(pagesize=pagesize, revid=revid, start_revid=start_revid, revid_list=revid_list,
+                                        file_id=file_id, scan_url='/changes', branch=self._branch, feed=True)
+            if query is not None:
+                navigation.query = query
+            util.fill_in_navigation(navigation)
+
+            entries = list(entries)
+            # add parent & merge-point branch-nick info, in case it's useful
+            h.get_branch_nicks(entries)
+
+            # does every change on this page have the same committer?  if so,
+            # tell the template to show committer info in the "details block"
+            # instead of on each line.
+            all_same_author = True
+
+            if entries:
+                author = entries[0].author
+                for e in entries[1:]:
+                    if e.author != author:
+                        all_same_author = False
+                        break
+
+            vals = {
+                'branch': self._branch,
+                'changes': entries,
+                'util': util,
+                'history': h,
+                'revid': revid,
+                'navigation': navigation,
+                'file_id': file_id,
+                'start_revid': start_revid,
+                'viewing_from': (orig_start_revid is not None) and (orig_start_revid != h.last_revid),
+                'query': query,
+                'search_failed': search_failed,
+                'all_same_author': all_same_author,
+            }
+            h.flush_cache()
+            self.log.info('/changes %r: %r secs' % (revid, time.time() - z))
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/download_ui.py'
--- loggerhead/controllers/download_ui.py	2007-01-24 19:26:27 +0000
+++ loggerhead/controllers/download_ui.py	2007-08-23 12:18:18 +0000
@@ -47,18 +47,23 @@
         z = time.time()
         h = self._branch.get_history()
         
-        if len(args) < 2:
-            raise HTTPRedirect(self._branch.url('/changes'))
-        
-        revid = h.fix_revid(args[0])
-        file_id = args[1]
-        path, filename, content = h.get_file(file_id, revid)
-        mime_type, encoding = mimetypes.guess_type(filename)
-        if mime_type is None:
-            mime_type = 'application/octet-stream'
-
-        self.log.info('/download %s @ %s (%d bytes)', path, h.get_revno(revid), len(content))
-        response.headers['Content-Type'] = mime_type
-        response.headers['Content-Length'] = len(content)
-        response.body = content
-        return response.body
+        h._branch.lock_read()
+        try:
+            if len(args) < 2:
+                raise HTTPRedirect(self._branch.url('/changes'))
+
+            revid = h.fix_revid(args[0])
+            file_id = args[1]
+            path, filename, content = h.get_file(file_id, revid)
+            mime_type, encoding = mimetypes.guess_type(filename)
+            if mime_type is None:
+                mime_type = 'application/octet-stream'
+
+            self.log.info('/download %s @ %s (%d bytes)', path, h.get_revno(revid), len(content))
+            response.headers['Content-Type'] = mime_type
+            response.headers['Content-Length'] = len(content)
+            response.headers['Content-Disposition'] = 'attachment; filename=%s'%(filename,)
+            response.body = content
+            return response.body
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2007-03-14 06:10:22 +0000
+++ loggerhead/controllers/inventory_ui.py	2007-08-22 22:39:43 +0000
@@ -52,57 +52,59 @@
         z = time.time()
         h = self._branch.get_history()
         util.set_context(kw)
-        
-        if len(args) > 0:
-            revid = h.fix_revid(args[0])
-        else:
-            revid = None
-        
-        file_id = kw.get('file_id', None)
-        sort_type = kw.get('sort', None)
 
+        h._branch.lock_read()
         try:
-            revid_list, revid = h.get_file_view(revid, file_id)
-            rev = h.get_revision(revid)
-            inv = h.get_inventory(revid)
-        except:
-            self.log.exception('Exception fetching changes')
-            raise InternalError('Could not fetch changes')
-
-        # no navbar for revisions
-        navigation = util.Container()
-
-        change = h.get_changes([ revid ])[0]
-        # add parent & merge-point branch-nick info, in case it's useful
-        h.get_branch_nicks([ change ])
-        
-        path = inv.id2path(file_id)
-        if not path.startswith('/'):
-            path = '/' + path
-        idpath = inv.get_idpath(file_id)
-        if len(idpath) > 1:
-            updir = dirname(path)
-            updir_file_id = idpath[-2]
-        else:
-            updir = None
-            updir_file_id = None
-        if updir == '/':
-            updir_file_id = None
-
-        vals = {
-            'branch': self._branch,
-            'util': util,
-            'revid': revid,
-            'change': change,
-            'file_id': file_id,
-            'path': path,
-            'updir': updir,
-            'updir_file_id': updir_file_id,
-            'filelist': h.get_filelist(inv, path, sort_type),
-            'history': h,
-            'posixpath': posixpath,
-            'navigation': navigation,
-        }
-        h.flush_cache()
-        self.log.info('/inventory %r: %r secs' % (revid, time.time() - z))
-        return vals
+            if len(args) > 0:
+                revid = h.fix_revid(args[0])
+            else:
+                revid = h.last_revid
+
+            try:
+                inv = h.get_inventory(revid)
+            except:
+                self.log.exception('Exception fetching changes')
+                raise InternalError('Could not fetch changes')
+
+            file_id = kw.get('file_id', inv.root.file_id)
+            sort_type = kw.get('sort', None)
+
+            # no navbar for revisions
+            navigation = util.Container()
+
+            change = h.get_changes([ revid ])[0]
+            # add parent & merge-point branch-nick info, in case it's useful
+            h.get_branch_nicks([ change ])
+
+            path = inv.id2path(file_id)
+            if not path.startswith('/'):
+                path = '/' + path
+            idpath = inv.get_idpath(file_id)
+            if len(idpath) > 1:
+                updir = dirname(path)
+                updir_file_id = idpath[-2]
+            else:
+                updir = None
+                updir_file_id = None
+            if updir == '/':
+                updir_file_id = None
+
+            vals = {
+                'branch': self._branch,
+                'util': util,
+                'revid': revid,
+                'change': change,
+                'file_id': file_id,
+                'path': path,
+                'updir': updir,
+                'updir_file_id': updir_file_id,
+                'filelist': h.get_filelist(inv, file_id, sort_type),
+                'history': h,
+                'posixpath': posixpath,
+                'navigation': navigation,
+            }
+            h.flush_cache()
+            self.log.info('/inventory %r: %r secs' % (revid, time.time() - z))
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2007-03-26 06:32:39 +0000
+++ loggerhead/controllers/revision_ui.py	2007-08-23 12:18:18 +0000
@@ -44,55 +44,56 @@
         h = self._branch.get_history()
         util.set_context(kw)
         
-        if len(args) > 0:
-            revid = h.fix_revid(args[0])
-        else:
-            revid = None
-        
-        file_id = kw.get('file_id', None)
-        start_revid = h.fix_revid(kw.get('start_revid', None))
-        query = kw.get('q', None)
-        remember = kw.get('remember', None)
-        compare_revid = kw.get('compare_revid', None)
-        
+        h._branch.lock_read()
         try:
-            revid, start_revid, revid_list = h.get_view(revid, start_revid, file_id, query)
-        except:
-            self.log.exception('Exception fetching changes')
-            raise InternalError('Could not fetch changes')
-        
-        navigation = util.Container(revid_list=revid_list, revid=revid, start_revid=start_revid, file_id=file_id,
-                                    pagesize=1, scan_url='/revision', branch=self._branch, feed=True)
-        if query is not None:
-            navigation.query = query
-        util.fill_in_navigation(h, navigation)
-
-        if compare_revid is not None:
-            change = h.get_diff(compare_revid, revid)
-        else:
-            change = h.get_changes([ revid ], get_diffs=True)[0]
-        # add parent & merge-point branch-nick info, in case it's useful
-        h.get_branch_nicks([ change ])
-
-        # let's make side-by-side diff be the default
-        side_by_side = not kw.get('unified', False)
-        if side_by_side:
-            h.add_side_by_side([ change ])
-        
-        vals = {
-            'branch': self._branch,
-            'revid': revid,
-            'change': change,
-            'start_revid': start_revid,
-            'file_id': file_id,
-            'util': util,
-            'history': h,
-            'navigation': navigation,
-            'query': query,
-            'remember': remember,
-            'compare_revid': compare_revid,
-            'side_by_side': side_by_side,
-        }
-        h.flush_cache()
-        self.log.info('/revision: %r seconds' % (time.time() - z,))
-        return vals
+            if len(args) > 0:
+                revid = h.fix_revid(args[0])
+            else:
+                revid = None
+
+            file_id = kw.get('file_id', None)
+            start_revid = h.fix_revid(kw.get('start_revid', None))
+            query = kw.get('q', None)
+            remember = kw.get('remember', None)
+            compare_revid = kw.get('compare_revid', None)
+
+            try:
+                revid, start_revid, revid_list = h.get_view(revid, start_revid, file_id, query)
+            except:
+                self.log.exception('Exception fetching changes')
+                raise InternalError('Could not fetch changes')
+
+            navigation = util.Container(revid_list=revid_list, revid=revid, start_revid=start_revid, file_id=file_id,
+                                        pagesize=1, scan_url='/revision', branch=self._branch, feed=True)
+            if query is not None:
+                navigation.query = query
+            util.fill_in_navigation(navigation)
+
+            change = h.get_change_with_diff(revid, compare_revid)
+            # add parent & merge-point branch-nick info, in case it's useful
+            h.get_branch_nicks([ change ])
+
+            # let's make side-by-side diff be the default
+            side_by_side = not kw.get('unified', False)
+            if side_by_side:
+                h.add_side_by_side([ change ])
+
+            vals = {
+                'branch': self._branch,
+                'revid': revid,
+                'change': change,
+                'start_revid': start_revid,
+                'file_id': file_id,
+                'util': util,
+                'history': h,
+                'navigation': navigation,
+                'query': query,
+                'remember': remember,
+                'compare_revid': compare_revid,
+                'side_by_side': side_by_side,
+            }
+            h.flush_cache()
+            self.log.info('/revision: %r seconds' % (time.time() - z,))
+            return vals
+        finally:
+            h._branch.unlock()

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2007-05-29 11:19:07 +0000
+++ loggerhead/history.py	2007-09-07 18:46:20 +0000
@@ -51,6 +51,7 @@
 import bzrlib.diff
 import bzrlib.errors
 import bzrlib.progress
+import bzrlib.revision
 import bzrlib.textfile
 import bzrlib.tsort
 import bzrlib.ui
@@ -58,18 +59,6 @@
 
 with_branch_lock = util.with_lock('_lock', 'branch')
 
-@decorator
-def with_bzrlib_read_lock(unbound):
-    def bzrlib_read_locked(self, *args, **kw):
-        #self.log.debug('-> %r bzr lock', id(threading.currentThread()))
-        self._branch.repository.lock_read()
-        try:
-            return unbound(self, *args, **kw)
-        finally:
-            self._branch.repository.unlock()
-            #self.log.debug('<- %r bzr lock', id(threading.currentThread()))
-    return bzrlib_read_locked
-
 
 # bzrlib's UIFactory is not thread-safe
 uihack = threading.local()
@@ -100,7 +89,7 @@
     """
     turn a normal unified-style diff (post-processed by parse_delta) into a
     side-by-side diff structure.  the new structure is::
-    
+
         chunks: list(
             diff: list(
                 old_lineno: int,
@@ -150,15 +139,24 @@
     elif len(message) == 0:
         # sometimes a commit may have NO message!
         message = ['']
-        
+
     # make short form of commit message
     short_message = message[0]
-    if len(short_message) > 60:
-        short_message = short_message[:60] + '...'
-    
+    if len(short_message) > 80:
+        short_message = short_message[:80] + '...'
+
     return message, short_message
 
 
+def rich_filename(path, kind):
+    if kind == 'directory':
+        path += '/'
+    if kind == 'symlink':
+        path += '@'
+    return path
+
+
+
 # from bzrlib
 class _RevListToTimestamps(object):
     """This takes a list of revisions, and allows you to bisect by date"""
@@ -178,50 +176,50 @@
 
 
 class History (object):
-    
+
     def __init__(self):
         self._change_cache = None
+        self._file_change_cache = None
         self._index = None
         self._lock = threading.RLock()
-    
+
     @classmethod
     def from_branch(cls, branch, name=None):
         z = time.time()
         self = cls()
         self._branch = branch
-        self._history = branch.revision_history()
-        self._last_revid = self._history[-1]
-        self._revision_graph = branch.repository.get_revision_graph(self._last_revid)
-        
+        self._last_revid = self._branch.last_revision()
+        if self._last_revid is not None:
+            self._revision_graph = branch.repository.get_revision_graph(self._last_revid)
+        else:
+            self._revision_graph = {}
+
         if name is None:
             name = self._branch.nick
         self._name = name
         self.log = logging.getLogger('loggerhead.%s' % (name,))
-        
+
         self._full_history = []
         self._revision_info = {}
         self._revno_revid = {}
         self._merge_sort = bzrlib.tsort.merge_sort(self._revision_graph, self._last_revid, generate_revno=True)
-        count = 0
         for (seq, revid, merge_depth, revno, end_of_merge) in self._merge_sort:
             self._full_history.append(revid)
             revno_str = '.'.join(str(n) for n in revno)
             self._revno_revid[revno_str] = revid
             self._revision_info[revid] = (seq, revid, merge_depth, revno_str, end_of_merge)
-            count += 1
-        self._count = count
 
         # cache merge info
         self._where_merged = {}
         for revid in self._revision_graph.keys():
-            if not revid in self._full_history: 
+            if not revid in self._full_history:
                 continue
             for parent in self._revision_graph[revid]:
                 self._where_merged.setdefault(parent, set()).add(revid)
 
         self.log.info('built revision graph cache: %r secs' % (time.time() - z,))
         return self
-    
+
     @classmethod
     def from_folder(cls, path, name=None):
         b = bzrlib.branch.Branch.open(path)
@@ -229,13 +227,18 @@
 
     @with_branch_lock
     def out_of_date(self):
-        if self._branch.revision_history()[-1] != self._last_revid:
+        # the branch may have been upgraded on disk, in which case we're stale.
+        if self._branch.__class__ is not \
+               bzrlib.branch.Branch.open(self._branch.base).__class__:
             return True
-        return False
+        return self._branch.last_revision() != self._last_revid
 
     def use_cache(self, cache):
         self._change_cache = cache
-    
+
+    def use_file_cache(self, cache):
+        self._file_change_cache = cache
+
     def use_search_index(self, index):
         self._index = index
 
@@ -257,25 +260,19 @@
         if self._change_cache is None:
             return
         self._change_cache.flush()
-    
+
     def check_rebuild(self):
         if self._change_cache is not None:
             self._change_cache.check_rebuild()
         if self._index is not None:
             self._index.check_rebuild()
-    
+
     last_revid = property(lambda self: self._last_revid, None, None)
-    
-    count = property(lambda self: self._count, None, None)
 
     @with_branch_lock
     def get_config(self):
         return self._branch.get_config()
-    
-    @with_branch_lock
-    def get_revision(self, revid):
-        return self._branch.repository.get_revision(revid)
-    
+
     def get_revno(self, revid):
         if revid not in self._revision_info:
             # ghost parent?
@@ -283,24 +280,9 @@
         seq, revid, merge_depth, revno_str, end_of_merge = self._revision_info[revid]
         return revno_str
 
-    def get_sequence(self, revid):
-        seq, revid, merge_depth, revno_str, end_of_merge = self._revision_info[revid]
-        return seq
-    
     def get_revision_history(self):
         return self._full_history
-    
-    def get_revid_sequence(self, revid_list, revid):
-        """
-        given a list of revision ids, return the sequence # of this revid in
-        the list.
-        """
-        seq = 0
-        for r in revid_list:
-            if revid == r:
-                return seq
-            seq += 1
-    
+
     def get_revids_from(self, revid_list, revid):
         """
         given a list of revision ids, yield revisions in graph order,
@@ -316,7 +298,7 @@
             if len(parents) == 0:
                 return
             revid = parents[0]
-    
+
     @with_branch_lock
     def get_short_revision_history_by_fileid(self, file_id):
         # wow.  is this really the only way we can get this list?  by
@@ -341,7 +323,7 @@
         revid_list.reverse()
         index = -index
         return revid_list[index:]
-    
+
     @with_branch_lock
     def get_revision_history_matching(self, revid_list, text):
         self.log.debug('searching %d revisions for %r', len(revid_list), text)
@@ -366,12 +348,12 @@
         # put them in some coherent order :)
         out = [r for r in self._full_history if r in out]
         return out
-    
+
     @with_branch_lock
     def get_search_revid_list(self, query, revid_list):
         """
         given a "quick-search" query, try a few obvious possible meanings:
-        
+
             - revision id or # ("128.1.3")
             - date (US style "mm/dd/yy", earth style "dd-mm-yy", or iso style "yyyy-mm-dd")
             - comment text as a fallback
@@ -381,14 +363,16 @@
         # FIXME: there is some silliness in this action.  we have to look up
         # all the relevant changes (time-consuming) only to return a list of
         # revids which will be used to fetch a set of changes again.
-        
+
         # if they entered a revid, just jump straight there; ignore the passed-in revid_list
         revid = self.fix_revid(query)
         if revid is not None:
+            if isinstance(revid, unicode):
+                revid = revid.encode('utf-8')
             changes = self.get_changes([ revid ])
             if (changes is not None) and (len(changes) > 0):
                 return [ revid ]
-        
+
         date = None
         m = self.us_date_re.match(query)
         if m is not None:
@@ -406,12 +390,12 @@
                 # if no limit to the query was given, search only the direct-parent path.
                 revid_list = list(self.get_revids_from(None, self._last_revid))
             return self.get_revision_history_since(revid_list, date)
-        
+
         # check comment fields.
         if revid_list is None:
             revid_list = self._full_history
         return self.get_revision_history_matching_indexed(revid_list, query)
-    
+
     revno_re = re.compile(r'^[\d\.]+$')
     # the date regex are without a final '$' so that queries like
     # "2006-11-30 12:15" still mostly work.  (i think it's better to give
@@ -429,66 +413,67 @@
         if self.revno_re.match(revid):
             revid = self._revno_revid[revid]
         return revid
-    
+
     @with_branch_lock
     def get_file_view(self, revid, file_id):
         """
-        Given an optional revid and optional path, return a (revlist, revid)
-        for navigation through the current scope: from the revid (or the
-        latest revision) back to the original revision.
-        
+        Given a revid and optional path, return a (revlist, revid) for
+        navigation through the current scope: from the revid (or the latest
+        revision) back to the original revision.
+
         If file_id is None, the entire revision history is the list scope.
-        If revid is None, the latest revision is used.
         """
         if revid is None:
             revid = self._last_revid
         if file_id is not None:
-            # since revid is 'start_revid', possibly should start the path tracing from revid... FIXME
-            inv = self._branch.repository.get_revision_inventory(revid)
+            # since revid is 'start_revid', possibly should start the path
+            # tracing from revid... FIXME
             revlist = list(self.get_short_revision_history_by_fileid(file_id))
             revlist = list(self.get_revids_from(revlist, revid))
         else:
             revlist = list(self.get_revids_from(None, revid))
-        if revid is None:
-            revid = revlist[0]
-        return revlist, revid
-    
+        return revlist
+
     @with_branch_lock
     def get_view(self, revid, start_revid, file_id, query=None):
         """
         use the URL parameters (revid, start_revid, file_id, and query) to
         determine the revision list we're viewing (start_revid, file_id, query)
         and where we are in it (revid).
-        
+
         if a query is given, we're viewing query results.
         if a file_id is given, we're viewing revisions for a specific file.
         if a start_revid is given, we're viewing the branch from a
             specific revision up the tree.
         (these may be combined to view revisions for a specific file, from
             a specific revision, with a specific search query.)
-            
+
         returns a new (revid, start_revid, revid_list, scan_list) where:
-        
+
             - revid: current position within the view
             - start_revid: starting revision of this view
             - revid_list: list of revision ids for this view
-        
+
         file_id and query are never changed so aren't returned, but they may
         contain vital context for future url navigation.
         """
+        if start_revid is None:
+            start_revid = self._last_revid
+
         if query is None:
-            revid_list, start_revid = self.get_file_view(start_revid, file_id)
+            revid_list = self.get_file_view(start_revid, file_id)
             if revid is None:
                 revid = start_revid
             if revid not in revid_list:
                 # if the given revid is not in the revlist, use a revlist that
                 # starts at the given revid.
-                revid_list, start_revid = self.get_file_view(revid, file_id)
+                revid_list= self.get_file_view(revid, file_id)
+                start_revid = revid
             return revid, start_revid, revid_list
-        
+
         # potentially limit the search
-        if (start_revid is not None) or (file_id is not None):
-            revid_list, start_revid = self.get_file_view(start_revid, file_id)
+        if file_id is not None:
+            revid_list = self.get_file_view(start_revid, file_id)
         else:
             revid_list = None
 
@@ -513,29 +498,24 @@
         if (len(path) > 0) and not path.startswith('/'):
             path = '/' + path
         return path
-    
+
     @with_branch_lock
     def get_file_id(self, revid, path):
         if (len(path) > 0) and not path.startswith('/'):
             path = '/' + path
         return self._branch.repository.get_revision_inventory(revid).path2id(path)
-    
-    def get_where_merged(self, revid):
-        try:
-            return self._where_merged[revid]
-        except:
-            return []
-    
+
+
     def get_merge_point_list(self, revid):
         """
         Return the list of revids that have merged this node.
         """
-        if revid in self._history:
+        if '.' not in self.get_revno(revid):
             return []
-        
+
         merge_point = []
         while True:
-            children = self.get_where_merged(revid)
+            children = self._where_merged.get(revid, [])
             nexts = []
             for child in children:
                 child_parents = self._revision_graph[child]
@@ -555,7 +535,7 @@
                 merge_point.extend(merge_point_next)
 
             revid = nexts[0]
-            
+
     def simplify_merge_point_list(self, revids):
         """if a revision is already merged, don't show further merge points"""
         d = {}
@@ -598,24 +578,28 @@
                     p.branch_nick = p_change_dict[p.revid].branch_nick
                 else:
                     p.branch_nick = '(missing)'
-    
+
     @with_branch_lock
-    def get_changes(self, revid_list, get_diffs=False):
+    def get_changes(self, revid_list):
         if self._change_cache is None:
-            changes = self.get_changes_uncached(revid_list, get_diffs)
+            changes = self.get_changes_uncached(revid_list)
         else:
-            changes = self._change_cache.get_changes(revid_list, get_diffs)
-        if changes is None:
+            changes = self._change_cache.get_changes(revid_list)
+        if len(changes) == 0:
             return changes
-        
+
         # some data needs to be recalculated each time, because it may
         # change as new revisions are added.
-        for i in xrange(len(revid_list)):
-            revid = revid_list[i]
-            change = changes[i]
-            merge_revids = self.simplify_merge_point_list(self.get_merge_point_list(revid))
+        for change in changes:
+            merge_revids = self.simplify_merge_point_list(self.get_merge_point_list(change.revid))
             change.merge_points = [util.Container(revid=r, revno=self.get_revno(r)) for r in merge_revids]
-        
+            change.revno = self.get_revno(change.revid)
+
+        parity = 0
+        for change in changes:
+            change.parity = parity
+            parity ^= 1
+
         return changes
 
     # alright, let's profile this sucka.
@@ -629,48 +613,43 @@
         self.log.info('lsprof complete!')
         return ret
 
-    def _get_deltas_for_revisions_with_trees(self, revisions):
+    def _get_deltas_for_revisions_with_trees(self, entries):
         """Produce a generator of revision deltas.
-        
+
         Note that the input is a sequence of REVISIONS, not revision_ids.
         Trees will be held in memory until the generator exits.
         Each delta is relative to the revision's lefthand predecessor.
         """
         required_trees = set()
-        for revision in revisions:
-            required_trees.add(revision.revision_id)
-            required_trees.update(revision.parent_ids[:1])
-        trees = dict((t.get_revision_id(), t) for 
+        for entry in entries:
+            required_trees.add(entry.revid)
+            required_trees.update([p.revid for p in entry.parents[:1]])
+        trees = dict((t.get_revision_id(), t) for
                      t in self._branch.repository.revision_trees(required_trees))
         ret = []
         self._branch.repository.lock_read()
         try:
-            for revision in revisions:
-                if not revision.parent_ids:
-                    old_tree = self._branch.repository.revision_tree(None)
+            for entry in entries:
+                if not entry.parents:
+                    old_tree = self._branch.repository.revision_tree(
+                        bzrlib.revision.NULL_REVISION)
                 else:
-                    old_tree = trees[revision.parent_ids[0]]
-                tree = trees[revision.revision_id]
-                ret.append((tree, old_tree, tree.changes_from(old_tree)))
+                    old_tree = trees[entry.parents[0].revid]
+                tree = trees[entry.revid]
+                ret.append(tree.changes_from(old_tree))
             return ret
         finally:
             self._branch.repository.unlock()
-    
+
     def entry_from_revision(self, revision):
         commit_time = datetime.datetime.fromtimestamp(revision.timestamp)
-        
+
         parents = [util.Container(revid=r, revno=self.get_revno(r)) for r in revision.parent_ids]
 
-        if len(parents) == 0:
-            left_parent = None
-        else:
-            left_parent = revision.parent_ids[0]
-        
         message, short_message = clean_message(revision.message)
 
         entry = {
             'revid': revision.revision_id,
-            'revno': self.get_revno(revision.revision_id),
             'date': commit_time,
             'author': revision.committer,
             'branch_nick': revision.properties.get('branch-nick', None),
@@ -682,43 +661,64 @@
         return util.Container(entry)
 
     @with_branch_lock
-    @with_bzrlib_read_lock
-    def get_changes_uncached(self, revid_list, get_diffs=False):
-        done = False
-        while not done:
-            try:
-                rev_list = self._branch.repository.get_revisions(revid_list)
-                done = True
-            except (KeyError, bzrlib.errors.NoSuchRevision), e:
-                # this sometimes happens with arch-converted branches.
-                # i don't know why. :(
-                self.log.debug('No such revision (skipping): %s', e)
-                revid_list.remove(e.revision)
-        
-        delta_list = self._get_deltas_for_revisions_with_trees(rev_list)
-        combined_list = zip(rev_list, delta_list)
-        
-        entries = []
-        for rev, (new_tree, old_tree, delta) in combined_list:
-            entry = self.entry_from_revision(rev)
-            entry.changes = self.parse_delta(delta, get_diffs, old_tree, new_tree)
-            entries.append(entry)
-        
-        return entries
-
-    @with_bzrlib_read_lock
-    def _get_diff(self, revid1, revid2):
-        rev_tree1 = self._branch.repository.revision_tree(revid1)
-        rev_tree2 = self._branch.repository.revision_tree(revid2)
+    def get_changes_uncached(self, revid_list):
+        # Because we may loop and call get_revisions multiple times (to throw
+        # out dud revids), we grab a read lock.
+        self._branch.lock_read()
+        try:
+            while True:
+                try:
+                    rev_list = self._branch.repository.get_revisions(revid_list)
+                except (KeyError, bzrlib.errors.NoSuchRevision), e:
+                    # this sometimes happens with arch-converted branches.
+                    # i don't know why. :(
+                    self.log.debug('No such revision (skipping): %s', e)
+                    revid_list.remove(e.revision)
+                else:
+                    break
+
+            return [self.entry_from_revision(rev) for rev in rev_list]
+        finally:
+            self._branch.unlock()
+
+    def get_file_changes_uncached(self, entries):
+        delta_list = self._get_deltas_for_revisions_with_trees(entries)
+
+        return [self.parse_delta(delta) for delta in delta_list]
+
+    @with_branch_lock
+    def get_file_changes(self, entries):
+        if self._file_change_cache is None:
+            return self.get_file_changes_uncached(entries)
+        else:
+            return self._file_change_cache.get_file_changes(entries)
+
+    def add_changes(self, entries):
+        changes_list = self.get_file_changes(entries)
+
+        for entry, changes in zip(entries, changes_list):
+            entry.changes = changes
+
+    @with_branch_lock
+    def get_change_with_diff(self, revid, compare_revid=None):
+        entry = self.get_changes([revid])[0]
+
+        if compare_revid is None:
+            if entry.parents:
+                compare_revid = entry.parents[0].revid
+            else:
+                compare_revid = 'null:'
+
+        rev_tree1 = self._branch.repository.revision_tree(compare_revid)
+        rev_tree2 = self._branch.repository.revision_tree(revid)
         delta = rev_tree2.changes_from(rev_tree1)
-        return rev_tree1, rev_tree2, delta
-    
-    def get_diff(self, revid1, revid2):
-        rev_tree1, rev_tree2, delta = self._get_diff(revid1, revid2)
-        entry = self.get_changes([ revid2 ], False)[0]
-        entry.changes = self.parse_delta(delta, True, rev_tree1, rev_tree2)
+
+        entry.changes = self.parse_delta(delta)
+
+        entry.changes.modified = self._parse_diffs(rev_tree1, rev_tree2, delta)
+
         return entry
-    
+
     @with_branch_lock
     def get_file(self, file_id, revid):
         "returns (path, filename, data)"
@@ -729,16 +729,12 @@
         if not path.startswith('/'):
             path = '/' + path
         return path, inv_entry.name, rev_tree.get_file_text(file_id)
-    
-    @with_branch_lock
-    def parse_delta(self, delta, get_diffs=True, old_tree=None, new_tree=None):
+
+    def _parse_diffs(self, old_tree, new_tree, delta):
         """
-        Return a nested data structure containing the changes in a delta::
-        
-            added: list((filename, file_id)),
-            renamed: list((old_filename, new_filename, file_id)),
-            deleted: list((filename, file_id)),
-            modified: list(
+        Return a list of processed diffs, in the format::
+
+            list(
                 filename: str,
                 file_id: str,
                 chunks: list(
@@ -750,88 +746,103 @@
                     ),
                 ),
             )
-        
-        if C{get_diffs} is false, the C{chunks} will be omitted.
+        """
+        process = []
+        out = []
+
+        for old_path, new_path, fid, kind, text_modified, meta_modified in delta.renamed:
+            if text_modified:
+                process.append((old_path, new_path, fid, kind))
+        for path, fid, kind, text_modified, meta_modified in delta.modified:
+            process.append((path, path, fid, kind))
+
+        for old_path, new_path, fid, kind in process:
+            old_lines = old_tree.get_file_lines(fid)
+            new_lines = new_tree.get_file_lines(fid)
+            buffer = StringIO()
+            if old_lines != new_lines:
+                try:
+                    bzrlib.diff.internal_diff(old_path, old_lines,
+                                              new_path, new_lines, buffer)
+                except bzrlib.errors.BinaryFile:
+                    diff = ''
+                else:
+                    diff = buffer.getvalue()
+            else:
+                diff = ''
+            out.append(util.Container(filename=rich_filename(new_path, kind), file_id=fid, chunks=self._process_diff(diff)))
+
+        return out
+
+    def _process_diff(self, diff):
+        # doesn't really need to be a method; could be static.
+        chunks = []
+        chunk = None
+        for line in diff.splitlines():
+            if len(line) == 0:
+                continue
+            if line.startswith('+++ ') or line.startswith('--- '):
+                continue
+            if line.startswith('@@ '):
+                # new chunk
+                if chunk is not None:
+                    chunks.append(chunk)
+                chunk = util.Container()
+                chunk.diff = []
+                lines = [int(x.split(',')[0][1:]) for x in line.split(' ')[1:3]]
+                old_lineno = lines[0]
+                new_lineno = lines[1]
+            elif line.startswith(' '):
+                chunk.diff.append(util.Container(old_lineno=old_lineno, new_lineno=new_lineno,
+                                                 type='context', line=util.fixed_width(line[1:])))
+                old_lineno += 1
+                new_lineno += 1
+            elif line.startswith('+'):
+                chunk.diff.append(util.Container(old_lineno=None, new_lineno=new_lineno,
+                                                 type='insert', line=util.fixed_width(line[1:])))
+                new_lineno += 1
+            elif line.startswith('-'):
+                chunk.diff.append(util.Container(old_lineno=old_lineno, new_lineno=None,
+                                                 type='delete', line=util.fixed_width(line[1:])))
+                old_lineno += 1
+            else:
+                chunk.diff.append(util.Container(old_lineno=None, new_lineno=None,
+                                                 type='unknown', line=util.fixed_width(repr(line))))
+        if chunk is not None:
+            chunks.append(chunk)
+        return chunks
+
+    def parse_delta(self, delta):
+        """
+        Return a nested data structure containing the changes in a delta::
+
+            added: list((filename, file_id)),
+            renamed: list((old_filename, new_filename, file_id)),
+            deleted: list((filename, file_id)),
+            modified: list(
+                filename: str,
+                file_id: str,
+            )
         """
         added = []
         modified = []
         renamed = []
         removed = []
-        
-        def rich_filename(path, kind):
-            if kind == 'directory':
-                path += '/'
-            if kind == 'symlink':
-                path += '@'
-            return path
-        
-        def process_diff(diff):
-            chunks = []
-            chunk = None
-            for line in diff.splitlines():
-                if len(line) == 0:
-                    continue
-                if line.startswith('+++ ') or line.startswith('--- '):
-                    continue
-                if line.startswith('@@ '):
-                    # new chunk
-                    if chunk is not None:
-                        chunks.append(chunk)
-                    chunk = util.Container()
-                    chunk.diff = []
-                    lines = [int(x.split(',')[0][1:]) for x in line.split(' ')[1:3]]
-                    old_lineno = lines[0]
-                    new_lineno = lines[1]
-                elif line.startswith(' '):
-                    chunk.diff.append(util.Container(old_lineno=old_lineno, new_lineno=new_lineno,
-                                                     type='context', line=util.html_clean(line[1:])))
-                    old_lineno += 1
-                    new_lineno += 1
-                elif line.startswith('+'):
-                    chunk.diff.append(util.Container(old_lineno=None, new_lineno=new_lineno,
-                                                     type='insert', line=util.html_clean(line[1:])))
-                    new_lineno += 1
-                elif line.startswith('-'):
-                    chunk.diff.append(util.Container(old_lineno=old_lineno, new_lineno=None,
-                                                     type='delete', line=util.html_clean(line[1:])))
-                    old_lineno += 1
-                else:
-                    chunk.diff.append(util.Container(old_lineno=None, new_lineno=None,
-                                                     type='unknown', line=util.html_clean(repr(line))))
-            if chunk is not None:
-                chunks.append(chunk)
-            return chunks
-                    
-        def handle_modify(old_path, new_path, fid, kind):
-            if not get_diffs:
-                modified.append(util.Container(filename=rich_filename(new_path, kind), file_id=fid))
-                return
-            old_lines = old_tree.get_file_lines(fid)
-            new_lines = new_tree.get_file_lines(fid)
-            buffer = StringIO()
-            try:
-                bzrlib.diff.internal_diff(old_path, old_lines,
-                                          new_path, new_lines, buffer)
-            except bzrlib.errors.BinaryFile:
-                diff = ''
-            else:
-                diff = buffer.getvalue()
-            modified.append(util.Container(filename=rich_filename(new_path, kind), file_id=fid, chunks=process_diff(diff), raw_diff=diff))
 
         for path, fid, kind in delta.added:
             added.append((rich_filename(path, kind), fid))
-        
+
         for path, fid, kind, text_modified, meta_modified in delta.modified:
-            handle_modify(path, path, fid, kind)
-        
-        for oldpath, newpath, fid, kind, text_modified, meta_modified in delta.renamed:
-            renamed.append((rich_filename(oldpath, kind), rich_filename(newpath, kind), fid))
+            modified.append(util.Container(filename=rich_filename(path, kind), file_id=fid))
+
+        for old_path, new_path, fid, kind, text_modified, meta_modified in delta.renamed:
+            renamed.append((rich_filename(old_path, kind), rich_filename(new_path, kind), fid))
             if meta_modified or text_modified:
-                handle_modify(oldpath, newpath, fid, kind)
-        
+                modified.append(util.Container(filename=rich_filename(new_path, kind), file_id=fid))
+
         for path, fid, kind in delta.removed:
             removed.append((rich_filename(path, kind), fid))
-        
+
         return util.Container(added=added, renamed=renamed, removed=removed, modified=modified)
 
     @staticmethod
@@ -840,47 +851,47 @@
         for change in changes:
             for m in change.changes.modified:
                 m.sbs_chunks = _make_side_by_side(m.chunks)
-    
+
     @with_branch_lock
-    def get_filelist(self, inv, path, sort_type=None):
+    def get_filelist(self, inv, file_id, sort_type=None):
         """
         return the list of all files (and their attributes) within a given
         path subtree.
         """
-        while path.endswith('/'):
-            path = path[:-1]
-        if path.startswith('/'):
-            path = path[1:]
-        
-        entries = inv.entries()
-        
+
+        dir_ie = inv[file_id]
+        path = inv.id2path(file_id)
         file_list = []
-        for filepath, entry in entries:
-            if posixpath.dirname(filepath) != path:
-                continue
-            filename = posixpath.basename(filepath)
-            rich_filename = filename
+
+        revid_set = set()
+
+        for filename, entry in dir_ie.children.iteritems():
+            revid_set.add(entry.revision)
+
+        change_dict = {}
+        for change in self.get_changes(list(revid_set)):
+            change_dict[change.revid] = change
+
+        for filename, entry in dir_ie.children.iteritems():
             pathname = filename
             if entry.kind == 'directory':
                 pathname += '/'
 
             revid = entry.revision
-            revision = self._branch.repository.get_revision(revid)
 
-            change = util.Container(date=datetime.datetime.fromtimestamp(revision.timestamp),
-                                    revno=self.get_revno(revid))
-            
-            file = util.Container(filename=filename, rich_filename=rich_filename, executable=entry.executable, kind=entry.kind,
-                                  pathname=pathname, file_id=entry.file_id, size=entry.text_size, revid=revid, change=change)
+            file = util.Container(
+                filename=filename, executable=entry.executable, kind=entry.kind,
+                pathname=pathname, file_id=entry.file_id, size=entry.text_size,
+                revid=revid, change=change_dict[revid])
             file_list.append(file)
-        
-        if sort_type == 'filename':
+
+        if sort_type == 'filename' or sort_type is None:
             file_list.sort(key=lambda x: x.filename)
         elif sort_type == 'size':
             file_list.sort(key=lambda x: x.size)
         elif sort_type == 'date':
             file_list.sort(key=lambda x: x.change.date)
-        
+
         parity = 0
         for file in file_list:
             file.parity = parity
@@ -896,26 +907,26 @@
         z = time.time()
         lineno = 1
         parity = 0
-        
+
         file_revid = self.get_inventory(revid)[file_id].revision
         oldvalues = None
-        
+
         # because we cache revision metadata ourselves, it's actually much
         # faster to call 'annotate_iter' on the weave directly than it is to
         # ask bzrlib to annotate for us.
         w = self._branch.repository.weave_store.get_weave(file_id, self._branch.repository.get_transaction())
-        
+
         revid_set = set()
         for line_revid, text in w.annotate_iter(file_revid):
             revid_set.add(line_revid)
             if self._BADCHARS_RE.match(text):
                 # bail out; this isn't displayable text
                 yield util.Container(parity=0, lineno=1, status='same',
-                                     text='<i>' + util.html_clean('(This is a binary file.)') + '</i>',
+                                     text='(This is a binary file.)',
                                      change=util.Container())
                 return
         change_cache = dict([(c.revid, c) for c in self.get_changes(list(revid_set))])
-        
+
         last_line_revid = None
         for line_revid, text in w.annotate_iter(file_revid):
             if line_revid == last_line_revid:
@@ -929,15 +940,14 @@
                 trunc_revno = change.revno
                 if len(trunc_revno) > 10:
                     trunc_revno = trunc_revno[:9] + '...'
-                
+
             yield util.Container(parity=parity, lineno=lineno, status=status,
-                                 change=change, text=util.html_clean(text))
+                                 change=change, text=util.fixed_width(text))
             lineno += 1
-        
+
         self.log.debug('annotate: %r secs' % (time.time() - z,))
 
     @with_branch_lock
-    @with_bzrlib_read_lock
     def get_bundle(self, revid, compare_revid=None):
         if compare_revid is None:
             parents = self._revision_graph[revid]

=== added file 'loggerhead/history2.py'
--- loggerhead/history2.py	1970-01-01 00:00:00 +0000
+++ loggerhead/history2.py	2007-06-25 10:16:34 +0000
@@ -0,0 +1,194 @@
+"""Extract data suitable for web presentation from Bazaar branches.
+
+The module is the interface between Loggerhead and bzrlib.  Other code in
+Loggerhead should not use bzrlib APIs, and functions and methods in this
+module should not return bzrlib data types.
+
+The core method is `History.getRevisionInfo`.  This method is meant to be
+cheap, so you should not worry about calling it multiple times with the same
+argument or calling it just to retreive a subset of the information it
+returns.
+
+Revisions are described using good old bzrlib revision ids.
+
+Revision ids, file ids, file names and file paths are always utf-8 encoded 8
+bit strings.
+"""
+
+class FileDelta(object):
+    """Information about how an inventory entry changed in a revision.
+
+    All attribute names are fairly self explanatory.  The ``*_parent`` fields
+    are the file id of the parent or ``None`` for the root.
+
+    :ivar file_id:
+    :ivar old_path:
+    :ivar new_path:
+    :ivar old_name:
+    :ivar new_name:
+    :ivar old_parent:
+    :ivar new_parent:
+    :ivar old_kind:
+    :ivar new_kind:
+    :ivar content_change:
+    :type content_change: ``bool``
+    :ivar execute_change: A boolean reflecting the new state if it changed, or
+                          ``None`` if it did not.
+    :type execute_change: ``bool`` or ``None``
+    """
+
+
+class FileChanges(object):
+    """Contains information about which files changed in a revision.
+
+    Note that the 'files changed' information is relative to left-most parent
+    by default, as in 'bzr log -v'.
+
+    :ivar deltas: A list of `FileDelta` objects.
+    :ivar fileset: ``set(d.file_id for d in self.deltas)``, but cached.
+    """
+
+class RevisionInfo(object):
+    """Contains all the immutable information Loggerhead needs about a
+    particular revision.
+
+    :ivar revid: Obvious.
+    :ivar date: The date and time this revision was committed.
+    :type date: ``datetime.datetime``
+    :ivar committer: The committer.
+    :type committer: utf-8 encoded ``str``
+    :ivar revprops: The branch properties.
+    :type revprops: A dictionary mapping ``str``\ s to ``str``\ s (both utf-8
+                    encoded).
+    :ivar message: The commit message of this revision.
+    :type message: utf-8 encoded ``str``
+    :ivar parents: The list of the revids of the revision's parents.
+    """
+
+class BranchRevisionInfo(object):
+    """Contains all the mutable information Loggerhead needs about a
+    particular revision, and points to the immutable data.
+
+    :ivar revno: The dotted revno of this revision in this branch.
+    :type revno: ``str``
+    :ivar where_merged: The list of revids that have this revision as a
+                        parent.
+    :ivar info: The corresponding `RevisionInfo`.
+    """
+
+class FileEntry(object):
+    """An entry in the list returned by `getFileList`.
+
+    :ivar path: The path of the object in the revision passed to
+                `getFileList`.
+    :ivar fileid: Obvious.
+    :ivar kind: one of ``'file'``, ``'executable'``, ``'link'`` or
+                ``'directory'``.
+    :type kind: ``str``
+    :ivar last_changed_revid: The revid of the revision in which this path
+                              last changed.  Not that this is recursive,
+                              i.e. the last_changed_revid for a directory is
+                              the revid in which the directory or anything
+                              contained in in changed.x
+    :ivar size: The size of this object in this revision in bytes.
+    :type size: ``int``
+    """
+
+class History(object):
+    """Provide the information loggerhead needs about a bzrlib Branch.
+
+    An instance of this class is effectively a wrapper around a
+    `bzrlib.branch.Branch` that translates the information provided by bzrlib
+    into a form convenient for use in loggerhead.
+
+    :ivar last_revision: The revid of the tip of the branch.
+    """
+
+    @classmethod
+    def fromBranch(cls, branch):
+        """Create and initialize a `History` object from a
+        `bzrlib.branch.Branch`.
+        """
+
+    def outOfDate(self):
+        """Decide whether this History object is still current.
+
+        At least to start with, an out of date History object should be thrown
+        away and a fresh one created.
+        """
+
+    def getRevisionInfo(self, revid):
+        """Find the `BranchRevisionInfo` object for a given revision id.
+
+        This operation is to be thought of as 'cheap', which is to say that
+        you should not worry about calling it repeatedly with the same
+        argument or about calling it just to retrieve a subset of the
+        information it includes.
+
+        :param revid: a revision id.
+        :returns: a `BranchRevisionInfo` object or ``None`` if the given
+                  revision id does not exist in the branch.
+        """
+
+    def getChanges(self, revid, other_revid=None):
+        """Find the `FileChanges` object between two revision.
+
+        :param revid: A revision id.
+        :param other_revid: The revision id to compare against.  If ``None``
+                            or not given, compare against the leftmost parent
+                            of ``revid``.
+        :returns: The corresponding `FileChanges` object.
+        """
+
+    # There will need to be some more methods here to allow searching for
+    # revisions by date, committer, commit message, etc...
+
+    def getDiff(self, revidA, revidB):
+        """Compute the diff between two revisions.
+
+        Not sure what this will return yet, probably something similar to but
+        more structured than the 'chunks' loggerhead already works with.
+        """
+
+    def getDirList(self, fileid, revid):
+        """List the files in the given directory as of the given revision.
+
+        :return: a list of `FileEntry` objects.
+        """
+
+    def getFileAnnotation(self, fileid, revid):
+        """Annotate the given file.
+
+        Not sure what this should return, mostly likely a list of
+        (revid-or-None, line contents) (and ``None`` for a binary file).
+        """
+
+    def getFileText(self, fileid, revid):
+        """Fetch the contents of a file at a particular revision.
+
+        :returns: (filename, data)
+        :rtype: ``(str, str)``
+        """
+
+    def getBundle(self, revid, compare_revid=None):
+        """Return a bundle for this revision.
+
+        Relative to compare_revid, or left-most parent if this is not supplied
+        or None.
+
+        :return: A bundle (in a string).
+        """
+
+    def normalizeRevivionSpec(self, revspec):
+        """Convert a revid or revno or 'head' to a revid.
+
+        :returns: a revision id.
+        """
+
+    def normalizeFileArguments(self, revid, fileid, filepath):
+        """Handle various ways of specifying a file in the branch.
+
+        One of ``fileid`` and ``filepath`` should be None.
+
+        :returns: (fileid, filepath)
+        """

=== modified file 'loggerhead/static/css/style.css'
--- loggerhead/static/css/style.css	2007-01-24 19:17:40 +0000
+++ loggerhead/static/css/style.css	2007-07-04 12:08:51 +0000
@@ -25,7 +25,7 @@
     color: #708a96;
     font-weight: normal;
     font-size: 1.6em;
-    margin-bottom: 1.5em;
+    margin-bottom: 1em;
 }
 h1 .filename {
     font-family: monospace;
@@ -201,68 +201,65 @@
 /* ----- changelog page ----- */
 
 /* secret css-jutsu to make hidden margins vanish on a table */
-.log-entries table {
+.log-entries {
     border-collapse: collapse;
-    width: 100%;
+    width: 98%;
+    margin-top: 0.5em;
+    margin-left: 1em;
+}
+
+.log-entries .log-header {
+    background-color: #8cacbb;
+    padding-top: 3px;
+    padding-bottom: 3px;
+    text-align: left;
 }
 
 .log-entries .revision-header {
-    background-color: #dee7ec;
-    padding: 3px;
-    width: 80%;
-    margin-top: 1em;
-    margin-left: 1em;
+    font-size: normal;
+}
+
+.log-entries .revision-header td {
+    padding-top: 2px;
+    padding-bottom: 2px;
 }
 
 .log-entries .revision-header td.expand-button {
     width: 20px;
 }
 
-.log-entries .revision-header .revision-number {
-    width: 6em;
+.log-entries .revision-number {
     padding-left: 0.5em;
 }
 
+.log-entries .revision-header td.date {
+    font-size: smaller;
+}
+
+.log-entries .revision-header td.author {
+    font-size: smaller;
+    padding-right: 0.5em;
+}
+
 .log-entries .revision-header .inventory-link {
     font-size: smaller;
     vertical-align: middle;
-    width: 4em;
-    text-align: right;
+    text-align: left;
     padding-right: 0.5em;
 }
 
-.log-entries .revision-details-block {
-    width: 80%;
-    border: none;
-    margin-left: 8em;
-    margin-bottom: 1.5em;
-}
-
-.log-entries .revision-details table {
+.log-entries table.revision-details {
     font-size: smaller;
-    width: 100%;
+    margin-bottom: 1.0em;
 }
 
-.log-entries .revision-details th {
-    width: 8em;
+.log-entries table.revision-details th {
     text-align: right;
     padding-right: 0.5em;
     vertical-align: top;
 }
 
-.log-entries .revision-details td.date {
-/*    font-family: monospace; */
-}
-
-.log-entries .revision-details th.files {
-    padding-top: 1em;
-}
-
-.log-entries .revision-details td.files {
-    padding-top: 1em;
-}
-
-.log-entries .revision-details td.files .filename {
+.log-entries table.revision-details td.files .filename {
     font-family: monospace;
 }
 
@@ -270,12 +267,8 @@
     font-size: smaller;
 }
 
-.log-entries .revision-details th.description {
-    padding-top: 1em;
-}
 .log-entries .revision-details td.description {
     font-family: monospace;
-    padding-top: 1em;
 }
 
 /* default state: hide all */
@@ -411,7 +404,7 @@
 .diff-option-buttons .collapse-file-showall { display: none; }
 
 .diff-block .show-button { display: none; }
-.diff .collapse-style-unified-content { display: none; }
+.collapse-style-unified-content { display: none; }
 .collapse-style-sbs-show { display: none; }
 
 .diff-insert {
@@ -470,7 +463,7 @@
 
 table.inventory {
     border-collapse: collapse;
-    width: 80%;
+    width: 100%;
     font-size: smaller;
     margin-top: 1em;
     border: 1px solid #8cacbb;
@@ -498,8 +491,8 @@
 
 .inventory td.permissions {
     font-family: monospace;
+    padding-left: 0.5em;
     width: 9em;
-    padding-left: 0.5em;
 }
 
 .inventory td.filename {
@@ -507,31 +500,30 @@
 }
 
 .inventory td.size {
-	font-family: monospace;
-	font-size: smaller;
-	width: 5em;
+    font-family: monospace;
+    font-size: smaller;
+    width: 5em;
 }
 
 .inventory td.revision {
+    padding-right: 1em;
+    text-align: left;
     width: 8em;
-    padding-right: 1.5em;
 }
 
 .inventory td.date {
-    font-family: monospace;
-    font-size: smaller;
+    padding-right: 1em;
     width: 10em;
-    padding-right: 1.5em;
 }
 
 .inventory td.changes-link {
+    font-size: smaller;
     width: 6em;
-    font-size: smaller;
 }
 
 .inventory td.download-link {
+    font-size: smaller;
     width: 8em;
-    font-size: smaller;
 }
 
 span.folder {
@@ -639,23 +631,23 @@
 }
 
 .browse-view td,.browse-view th {
-	padding: 0.25em 0.5em 0.25em 0.5em;
+    padding: 0.25em 0.5em 0.25em 0.5em;
 }
 
 .browse-view td.name {
-	width: 20%;
+    width: 20%;
 }
 
 .browse-view td.last-update {
-	font-size: smaller;
-	width: 15%;
+    font-size: smaller;
+    width: 20em;
 }
 
 .browse-view td.url {
-	font-family: monospace;
-	font-size: smaller;
-	padding-left: 3em;
-	padding-top: 0px;
+    font-family: monospace;
+    font-size: smaller;
+    padding-left: 3em;
+    padding-top: 0px;
 }
 
 .browse-view td.feed {

=== modified file 'loggerhead/templates/annotate.kid'
--- loggerhead/templates/annotate.kid	2007-05-21 08:06:44 +0000
+++ loggerhead/templates/annotate.kid	2007-06-21 16:39:33 +0000
@@ -15,7 +15,7 @@
 	    <div> <b>&#8594;</b> <a href="${branch.url([ '/files', revid ], **util.get_context(clear=1))}"> browse files </a> </div>
 	    <div> <b>&#8594;</b> <a href="${branch.url('/revision', **util.get_context(clear=1, start_revid=revid))}"> view revision </a> </div>
 	    <div> <b>&#8594;</b> <a href="${branch.url('/changes', **util.get_context(clear=1, start_revid=revid, file_id=file_id))}"> view changes to this file </a> </div>
-	    <div> <b>&#8594;</b> <a href="${branch.url([ '/download', revid, file_id, filename ])}"> view/download file </a> </div>
+	    <div> <b>&#8594;</b> <a href="${branch.url([ '/download', revid, file_id, filename ])}"> download file </a> </div>
 	</div>
 </h1>
 
@@ -33,7 +33,7 @@
                 <a py:if="line.status=='changed'" href="${branch.url('/revision', **util.get_context(clear=1, start_revid=line.change.revid, file_id=file_id))}"
                     title="${line.change.revno} by ${util.hide_email(line.change.author)}, on ${line.change.date.strftime('%d %b %Y %H:%M')} (${util.ago(line.change.date)})">${util.trunc(line.change.revno)}</a>
             </td>
-            <td class="text ${line.status}"> ${XML(line.text)} </td>
+            <td class="text ${line.status}"> ${line.text} </td>
         </tr>
     </table>
 </div>

=== modified file 'loggerhead/templates/browse.kid'
--- loggerhead/templates/browse.kid	2007-01-21 00:05:52 +0000
+++ loggerhead/templates/browse.kid	2007-06-08 11:52:33 +0000
@@ -34,7 +34,7 @@
             <tr>
             <td class="name"> <a href="${tg.url([ '/' + project.name, view.name ])}">${view.friendly_name}</a> </td>
             <td class="description"> ${view.description} </td>
-            <td class="last-update"> ${util.ago(view.last_updated())} </td>
+            <td class="last-update"> ${view.get_history().last_revid and util.ago(view.last_updated()) or "(no revisions)"} </td>
             <td class="feed">
                 <a href="${tg.url([ '/' + project.name, view.name, 'atom' ])}" title="atom feed">
                     <img src="${tg.url('/static/images/feed-icon-16x16.gif')}" alt="atom feed" />

=== modified file 'loggerhead/templates/changelog.kid'
--- loggerhead/templates/changelog.kid	2007-01-21 00:05:52 +0000
+++ loggerhead/templates/changelog.kid	2007-06-26 05:34:28 +0000
@@ -31,85 +31,100 @@
     Sorry, no results found for your search.
 </span>
 
+<span py:if="not changes">
+    No revisions!
+</span>
+
 <span py:if="not search_failed" class="changelog"> ${collapse_all_button('cl')} </span>
 
-<div class="log-entries">
+<table class="log-entries">
+    <tr class="log-header">
+        <th class="revision-number">Rev</th>
+        <th></th>
+        <th class="summary">Summary</th>
+        <span py:strip="True" py:if="all_same_author">
+            <th></th>
+        </span>
+        <span py:strip="True" py:if="not all_same_author">
+            <th class="author">Committer</th>
+        </span>
+        <th class="date" colspan="2">Date</th>
+    </tr>
     <div py:for="entry in changes" class="revision">
         <a name="entry-${entry.revno}" />
-        <div class="revision-header">
-            <table>
+        <tr class="revision-header parity${entry.parity}">
+            <td class="revision-number"> ${revision_link(entry.revid, util.trunc(entry.revno))} </td>
+            <td class="expand-button"> ${collapse_button('cl', entry.revno)} </td>
+            <td class="summary"> ${revision_link(entry.revid, entry.short_comment)} </td>
+            <span py:strip="True" py:if="all_same_author">
+                <td></td>
+            </span>
+            <span py:strip="True" py:if="not all_same_author">
+                <td class="author"> ${util.trunc(util.hide_email(entry.author), 20)} </td>
+            </span>
+            <td class="date"> ${entry.date.strftime('%Y-%m-%d, %H:%M')} &nbsp; (${util.ago(entry.date)}) </td>
+            <td class="inventory-link"> 
+                <a href="${branch.url([ '/files', entry.revid ])}"
+                    title="Files at revision ${entry.revno}"> files</a>
+            </td>
+        </tr>
+        
+        <tr class="revision-details-block parity${entry.parity}">
+            <td colspan="2"></td>
+            <td colspan="4"><table class="revision-details hidden-details collapse-cl-${entry.revno}-content">
+                <tr py:if="len(entry.merge_points) > 0">
+                    <th class="children"> merged in: </th>
+                    <td class="children">
+                        <span py:for="child in entry.merge_points">
+                            ${loglink(child.revid, '(' + child.revno + util.if_present(' %s', child.branch_nick) + ')')} <br />
+                        </span>
+                    </td>
+                </tr>
+                <tr py:if="len(entry.parents) > 1">
+                    <th class="parents"> merged from: </th>
+                    <td class="parents">
+                        <span py:for="parent in entry.parents"><span py:if="parent.revid != entry.parents[0].revid">
+                            ${loglink(parent.revid, '(' + parent.revno + util.if_present(' %s', parent.branch_nick) + ')')} <br />
+                        </span></span>
+                    </td>
+                </tr>
+                <span py:strip="True" py:if="all_same_author">
+                    <tr>
+                        <th class="author">committed by:</th>
+                        <td class="author"> ${util.hide_email(entry.author)} </td>
+                    </tr>
+                </span>
                 <tr>
-                    <td class="revision-number"> ${revision_link(entry.revid, util.trunc(entry.revno))} </td>
-                    <td class="expand-button"> ${collapse_button('cl', entry.revno)} </td>
-					<td class="summary"> ${revision_link(entry.revid, entry.short_comment)} </td>
-					<td class="inventory-link"> <a href="${branch.url([ '/files', entry.revid ])}">&#8594; files</a> </td>
-				</tr>
-			</table>
-        </div>
-        
-        <div class="revision-details-block">
-            <div class="revision-details">
-			    <table>
-			        <tr>
-			            <th class="author">committed by:</th>
-			            <td class="author"> ${util.hide_email(entry.author)} </td>
-			        </tr>
-			        <tr>
-			            <th class="date">date:</th>
-			            <td class="date"> ${entry.date.strftime('%d %b %Y %H:%M')} &nbsp; (${util.ago(entry.date)}) </td>
-			        </tr>
-			    </table>
-			</div>
-	        <div class="revision-details hidden-details collapse-cl-${entry.revno}-content">
-		        <table>
-			        <tr py:if="len(entry.merge_points) > 0">
-			            <th class="children"> merged in: </th>
-			            <td class="children">
-			                <span py:for="child in entry.merge_points">
-			                    ${loglink(child.revid, '(' + child.revno + util.if_present(' %s', child.branch_nick) + ')')} <br />
-			                </span>
-			            </td>
-			        </tr>
-			        <tr py:if="len(entry.parents) > 1">
-			        	<th class="parents"> merged from: </th>
-			        	<td class="parents">
-			        	    <span py:for="parent in entry.parents"><span py:if="parent.revid != entry.parents[0].revid">
-			        	        ${loglink(parent.revid, '(' + parent.revno + util.if_present(' %s', parent.branch_nick) + ')')} <br />
-			        	    </span></span>
-			        	</td>
-			        </tr>
-
-			        <tr py:if="entry.changes.added">
-			            <th class="files"> files added: </th>
-			            <td class="files"> <span py:for="filename, fid in entry.changes.added" class="filename">${file_link(filename, fid, entry.revid)} <br /></span> </td>
-			        </tr>
-			        <tr py:if="entry.changes.removed">
-			            <th class="files"> files removed: </th>
-			            <td class="files"> <span py:for="filename, fid in entry.changes.removed" class="filename">${file_link(filename, fid, entry.revid)} <br /></span> </td>
-			        </tr>
-			        <tr py:if="entry.changes.renamed">
-			            <th class="files"> files renamed: </th>
-			            <td class="files"> <span py:for="old_filename, new_filename, fid in entry.changes.renamed" class="filename">
-			                ${file_link(old_filename, fid, entry.revid)} => ${file_link(new_filename, fid, entry.revid)}<br />
-			            </span> </td>
-			        </tr>
-			        <tr py:if="entry.changes.modified">
-			            <th class="files"> files modified: </th>
-			            <td class="files"> <span py:for="item in entry.changes.modified">
-			                <span class="filename">${file_link(item.filename, item.file_id, entry.revid)}</span> &nbsp;
-			                <a href="${branch.url([ '/revision', entry.revid ], **util.get_context()) + '#' + item.filename}" class="jump">&#8594; diff</a>
-			                <br />
-			            </span> </td>
-			        </tr>
-			        <tr>
-			            <th class="description">description:</th>
-		                <td class="description"><span py:for="line in entry.comment_clean">${XML(line)} <br /></span> </td>
-		            </tr>
-		        </table>
-	    	</div>
-        </div>
+                    <th class="description">description:</th>
+                    <td class="description"><span py:for="line in entry.comment_clean">${XML(line)} <br /></span> </td>
+                </tr>
+                <tr py:if="entry.changes.added">
+                    <th class="files"> files added: </th>
+                    <td class="files"> <span py:for="filename, fid in entry.changes.added" class="filename">${file_link(filename, fid, entry.revid)} <br /></span> </td>
+                </tr>
+                <tr py:if="entry.changes.removed">
+                    <th class="files"> files removed: </th>
+                    <td class="files"> <span py:for="filename, fid in entry.changes.removed" class="filename">${file_link(filename, fid, entry.revid)} <br /></span> </td>
+                </tr>
+                <tr py:if="entry.changes.renamed">
+                    <th class="files"> files renamed: </th>
+                    <td class="files"> <span py:for="old_filename, new_filename, fid in entry.changes.renamed" class="filename">
+                        ${file_link(old_filename, fid, entry.revid)} => ${file_link(new_filename, fid, entry.revid)}<br />
+                    </span> </td>
+                </tr>
+                <tr py:if="entry.changes.modified">
+                    <th class="files"> files modified: </th>
+                    <td class="files"> <span py:for="item in entry.changes.modified">
+                        <span class="filename">${file_link(item.filename, item.file_id, entry.revid)}</span> &nbsp;
+                        <a href="${branch.url([ '/revision', entry.revid ], **util.get_context()) + '#' + item.filename}" class="jump">&#8594; diff</a>
+                        <br />
+                    </span> </td>
+                </tr>
+                </table>
+            </td>
+        </tr>
     </div>
-</div>
+</table>
 
 <div py:if="navigation.prev_page_revid or navigation.next_page_revid" class="bar">
     <table>

=== modified file 'loggerhead/templates/inventory.kid'
--- loggerhead/templates/inventory.kid	2007-01-24 19:26:50 +0000
+++ loggerhead/templates/inventory.kid	2007-06-25 06:44:52 +0000
@@ -63,11 +63,11 @@
     <tr class="header">
         <th class="permissions"> Permissions </th>
         <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='filename'))}">Filename</a> </th>
+        <th> Latest Rev </th>
+        <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='date'))}">Last Changed</a> </th>
         <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='size'))}">Size</a> </th>
-        <th> Last change </th>
-        <th> <a href="${branch.url([ '/files', revid ], **util.get_context(sort='date'))}">When</a> </th>
-        <th> History </th>
-        <th> Download </th>
+        <th>  </th>
+        <th>  </th>
     </tr>
     
     <tr class="parity1" py:if="updir">
@@ -86,16 +86,16 @@
             <a py:if="file.kind=='file'" href="${branch.url([ '/annotate', revid ],
                 **util.get_context(file_id=file.file_id))}" title="Annotate ${file.filename}">${file.filename}</a>
         </td>
+        <td class="revision"> ${revision_link(file.revid, util.trunc(file.change.revno, 15), **util.get_context(start_revid=file.revid, file_id=file.file_id))} </td>
+        <td class="date"> ${file.change.date.strftime('%Y-%m-%d, %H:%M')} </td>
         <td class="size"> <span py:if="file.kind=='file'"> ${util.human_size(file.size)} </span></td>
-        <td class="revision"> ${revision_link(file.revid, util.trunc(file.change.revno, 15), **util.get_context(start_revid=file.revid, file_id=file.file_id))} </td>
-        <td class="date"> ${file.change.date.strftime('%d %b %Y %H:%M')} </td>
         <td class="changes-link"> 
             <a href="${branch.url('/changes', **util.get_context(start_revid=file.revid, file_id=file.file_id))}"
-               title="Changes from ${file.change.revno} affecting ${file.filename}"> &#8594; changes </a>
+               title="Changes affecting ${file.filename} up to revision ${file.change.revno}"> changes </a>
         </td>
         <td class="download-link">
             <a href="${branch.url([ '/download', file.revid, file.file_id, file.filename ])}"
-               title="Download ${file.filename} at revision ${file.change.revno}"> &#8594; download </a></td>
+               title="Download ${file.filename} at revision ${file.change.revno}">  download </a></td>
     </tr>
 </table>
 

=== modified file 'loggerhead/templates/revision.kid'
--- loggerhead/templates/revision.kid	2007-01-24 19:26:50 +0000
+++ loggerhead/templates/revision.kid	2007-06-26 06:28:36 +0000
@@ -9,6 +9,10 @@
         <a href="${branch.url([ '/annotate', revid ],
             **util.get_context(file_id=file_id))}" title="Annotate ${filename}">${filename}</a>
     </span>
+    <span py:strip="True" py:def="file_link_rev(filename, file_id, revid)">
+        <a href="${branch.url([ '/annotate', revid ],
+            **util.get_context(file_id=file_id))}" title="Annotate ${filename} at revision ${history.get_revno(revid)}">${filename}</a>
+    </span>
     
     <span py:replace="use_collapse_buttons()"></span>
     
@@ -109,7 +113,9 @@
         </tr>
         <tr py:if="change.changes.removed">
             <th class="files"> files removed: </th>
-            <td class="files"> <span py:for="filename, file_id in change.changes.removed" class="filename">${file_link(filename, file_id)} <br /></span> </td>
+            <td class="files"> <span py:for="filename, file_id in change.changes.removed" class="filename">
+                ${file_link_rev(filename, file_id, change.parents[0].revid)} <br /></span>
+            </td>
         </tr>
         <tr py:if="change.changes.renamed">
             <th class="files"> files renamed: </th>
@@ -120,8 +126,11 @@
         <tr py:if="change.changes.modified">
             <th class="files"> files modified: </th>
             <td class="files">
-                <span py:for="item in change.changes.modified">
-                    <span class="filename">${file_link(item.filename, item.file_id)}</span> &nbsp; <a href="#${item.filename}" class="jump">&#8594; diff</a><br />
+                <span py:for="item in change.changes.modified" class="collapse-style-sbs-content">
+                    <a href="#${item.filename}-s" class="filename" title="Jump to ${item.filename} below">${item.filename}</a><br />
+                </span>
+                <span py:for="item in change.changes.modified" class="collapse-style-unified-content">
+                    <a href="#${item.filename}-u" class="filename" title="Jump to ${item.filename} below">${item.filename}</a><br />
                 </span>
             </td>
         </tr>
@@ -155,7 +164,8 @@
         <span py:strip="True" py:for="item in change.changes.modified">
             <tr><th class="filename" colspan="4">
                 ${collapse_button('file', util.uniq(uniqs, item.file_id), 'table-row')}
-                <a href="${branch.url([ '/annotate', change.revid ], **util.get_context(file_id=item.file_id))}" name="${item.filename}">${item.filename}</a>
+                <a href="${branch.url([ '/annotate', change.revid ], **util.get_context(file_id=item.file_id))}"
+                    name="${item.filename}-s" title="Annotate ${item.filename}">${item.filename}</a>
             </th></tr>
 
             <span py:strip="True" py:for="chunk in item.sbs_chunks">
@@ -165,19 +175,19 @@
                 <tr py:for="line in chunk.diff" class="diff-chunk collapse-file-${util.uniq(uniqs, item.file_id)}-content">
                     <span py:if="line.old_lineno" py:strip="True">
                         <td class="lineno">${line.old_lineno}</td>
-                        <td class="diff-${line.old_type}">${XML(line.old_line)}</td>
+                        <td class="diff-${line.old_type}">${line.old_line}</td>
                     </span>
                     <span py:if="not line.old_lineno" py:strip="True">
                         <td class="lineno-skip">${line.old_lineno}</td>
-                        <td class="diff-${line.old_type}-skip">${XML(line.old_line)}</td>
+                        <td class="diff-${line.old_type}-skip">${line.old_line}</td>
                     </span>
                     <span py:if="line.new_lineno" py:strip="True">
                         <td py:if="line.new_lineno" class="lineno">${line.new_lineno}</td>
-                        <td class="diff-${line.new_type}">${XML(line.new_line)}</td>
+                        <td class="diff-${line.new_type}">${line.new_line}</td>
                     </span>
                     <span py:if="not line.new_lineno" py:strip="True">
                         <td py:if="not line.new_lineno" class="lineno-skip">${line.new_lineno}</td>
-                        <td class="diff-${line.new_type}-skip">${XML(line.new_line)}</td>
+                        <td class="diff-${line.new_type}-skip">${line.new_line}</td>
                     </span>
                 </tr>
                 <tr class="diff-chunk-spacing collapse-file-${util.uniq(uniqs, item.file_id)}-content"> <td colspan="4"> &nbsp; </td> </tr>
@@ -188,24 +198,25 @@
     
     <!-- ! unified diff -->
     <table class="diff-block collapse-style-unified-content">
-	    <span py:strip="True" py:for="item in change.changes.modified">
-	        <tr><th class="filename" colspan="4">
-	            ${collapse_button('file', util.uniq(uniqs, item.file_id), 'table-row')}
-	            <a href="${branch.url([ '/annotate', change.revid ], **util.get_context(file_id=item.file_id))}" name="${item.filename}">${item.filename}</a>
-	        </th></tr>
-	
-	        <span py:strip="True" py:for="chunk in item.chunks">
-	            <tr class="diff-chunk collapse-file-${util.uniq(uniqs, item.file_id)}-content"> <th class="lineno">old</th> <th class="lineno">new</th> <th></th> <th></th> </tr>
-	            <tr py:for="line in chunk.diff" class="diff-chunk collapse-file-${util.uniq(uniqs, item.file_id)}-content">
-	                <td class="lineno">${line.old_lineno}</td>
-	                <td class="lineno">${line.new_lineno}</td>
-	                <td class="diff-${line.type} text">${XML(line.line)}</td>
-	                <td> </td>
-	            </tr>
-	            <tr class="diff-chunk-spacing collapse-file-${util.uniq(uniqs, item.file_id)}-content"> <td colspan="4"> &nbsp; </td> </tr>
-	        </span>
-	        <tr class="diff-spacing"> <td colspan="4"> &nbsp; </td> </tr>
-	    </span>
+        <span py:strip="True" py:for="item in change.changes.modified">
+            <tr><th class="filename" colspan="4">
+                ${collapse_button('file', util.uniq(uniqs, item.file_id), 'table-row')}
+                <a href="${branch.url([ '/annotate', change.revid ], **util.get_context(file_id=item.file_id))}"
+                    name="${item.filename}-u" title="Annotate ${item.filename}">${item.filename}</a>
+            </th></tr>
+
+            <span py:strip="True" py:for="chunk in item.chunks">
+                <tr class="diff-chunk collapse-file-${util.uniq(uniqs, item.file_id)}-content"> <th class="lineno">old</th> <th class="lineno">new</th> <th></th> <th></th> </tr>
+                <tr py:for="line in chunk.diff" class="diff-chunk collapse-file-${util.uniq(uniqs, item.file_id)}-content">
+                    <td class="lineno">${line.old_lineno}</td>
+                    <td class="lineno">${line.new_lineno}</td>
+                    <td class="diff-${line.type} text">${line.line}</td>
+                    <td> </td>
+                </tr>
+                <tr class="diff-chunk-spacing collapse-file-${util.uniq(uniqs, item.file_id)}-content"> <td colspan="4"> &nbsp; </td> </tr>
+            </span>
+            <tr class="diff-spacing"> <td colspan="4"> &nbsp; </td> </tr>
+        </span>
     </table>
 
 </div>

=== added file 'loggerhead/tests/test_corners.py'
--- loggerhead/tests/test_corners.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_corners.py	2007-06-27 15:38:47 +0000
@@ -0,0 +1,53 @@
+from loggerhead.tests.test_simple import BasicTests
+
+import os
+from turbogears import testutil
+import cherrypy
+
+class TestCornerCases(BasicTests):
+
+    def test_survive_over_upgrade(self):
+        self.createBranch()
+
+        f = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
+        try:
+            f.write("foo")
+        finally:
+            f.close()
+        self.tree.add('myfilename')
+        msg = 'a very exciting commit message'
+        self.tree.commit(message=msg)
+
+        self.setUpLoggerhead()
+
+        testutil.create_request('/project/branch/changes')
+        assert msg in cherrypy.response.body[0]
+
+        from bzrlib.upgrade import upgrade
+        from bzrlib.bzrdir import format_registry
+        upgrade(self.bzrbranch, format_registry.make_bzrdir('dirstate-tags'))
+
+        testutil.create_request('/project/branch/changes')
+        assert msg in cherrypy.response.body[0]
+
+    def test_revision_only_changing_execute_bit(self):
+        self.createBranch()
+
+        f = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
+        try:
+            f.write("foo")
+        finally:
+            f.close()
+        self.tree.add('myfilename')
+        msg = 'a very exciting commit message'
+        self.tree.commit(message=msg)
+
+        os.chmod(os.path.join(self.bzrbranch, 'myfilename'), 0755)
+
+        newrevid = self.tree.commit(message='make something executable')
+
+        self.setUpLoggerhead()
+
+
+        testutil.create_request('/project/branch/revision/'+newrevid)
+        assert 'executable' in cherrypy.response.body[0]

=== added file 'loggerhead/tests/test_filechangecache.py'
--- loggerhead/tests/test_filechangecache.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_filechangecache.py	2007-09-07 08:09:52 +0000
@@ -0,0 +1,95 @@
+"""Tests for the FileChangeCache in loggerhead.changecache."""
+
+import shutil
+import tempfile
+
+from loggerhead.changecache import FileChangeCache
+
+class MockEntry:
+    def __init__(self, revid):
+        self.revid = revid
+
+
+class MockHistory:
+    def __init__(self):
+        self.fetched_revids = set()
+    def get_file_changes_uncached(self, entries):
+        output = []
+        for entry in entries:
+            self.fetched_revids.add(entry.revid)
+            output.append(entry.revid)
+        return output
+
+
+class TestFileChangeCache(object):
+
+    # setup_method and teardown_method are so i can run the tests with
+    # py.test and take advantage of the error reporting.
+    def setup_method(self, meth):
+        self.setUp()
+
+    def teardown_method(self, meth):
+        self.tearDown()
+
+    def setUp(self):
+        self.cache_folders = []
+
+    def tearDown(self):
+        for folder in self.cache_folders:
+            shutil.rmtree(folder)
+
+
+    def makeHistoryAndEntriesForRevids(self, revids, fill_cache_with=[]):
+        cache_folder = tempfile.mkdtemp()
+        self.cache_folders.append(cache_folder)
+        self.history = MockHistory()
+        self.cache = FileChangeCache(self.history, cache_folder)
+
+        self.entries = [MockEntry(revid) for revid in revids]
+
+        self.cache.get_file_changes([entry for entry in self.entries
+                                     if entry.revid in fill_cache_with])
+        self.history.fetched_revids.clear()
+
+
+    def test_empty_cache(self):
+        """An empty cache passes all the revids through to the history object.
+        """
+        revids = ['a', 'b']
+        self.makeHistoryAndEntriesForRevids(revids)
+
+        result = self.cache.get_file_changes(self.entries)
+
+        assert result == self.revids
+        assert self.history.fetched_revids == set(self.revids)
+
+    def test_full_cache(self):
+        """A full cache passes none of the revids through to the history
+        object.
+        """
+        revids = ['a', 'b']
+        self.makeHistoryAndEntriesForRevids(revids, fill_cache_with=revids)
+
+        result = self.cache.get_file_changes(self.entries)
+
+        assert result == revids
+        assert self.history.fetched_revids == set()
+
+    def test_partial_cache(self):
+        """A partially full cache passes some of the revids through to the
+        history object, and preserves the ordering of the argument list.
+        """
+        # To test the preservation of argument order code, we put the uncached
+        # revid at the beginning, middle and then end of the list of revids
+        # being asked for.
+        for i in range(3):
+            cached_revids = ['a', 'b']
+            revids = cached_revids[:]
+            revids.insert(i, 'uncached')
+            self.makeHistoryAndEntriesForRevids(
+                revids, fillCacheWith=cached_revids)
+
+            result = self.cache.get_file_changes(self.entries)
+            assert result == revids
+
+            assert self.history.fetched_revids == set(['uncached'])

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2007-05-29 12:26:42 +0000
+++ loggerhead/tests/test_simple.py	2007-09-07 18:46:20 +0000
@@ -1,8 +1,10 @@
 from configobj import ConfigObj
+import cgi
 import unittest
 import os
 import tempfile
 import shutil
+import logging
 
 import cherrypy
 from turbogears import testutil
@@ -18,48 +20,66 @@
     testutil.create_request('/')
     assert 'loggerhead branches' in cherrypy.response.body[0]
 
-config_template = """
-[project]
-    [[branch]]
-        branch_name = 'branch'
-        folder = '%s'
-"""
-
-
-class TestWithSimpleTree(object):
+
+class BasicTests(object):
+
+    # setup_method and teardown_method are so i can run the tests with
+    # py.test and take advantage of the error reporting.
+    def setup_method(self, meth):
+        self.setUp()
+
+    def teardown_method(self, meth):
+        self.tearDown()
+
     def setUp(self):
+        #logging.basicConfig(level=logging.DEBUG)
+        self.bzrbranch = None
+        self.old_bzrhome = None
+
+    def createBranch(self):
         self.old_bzrhome = bzrlib.osutils.set_or_unset_env('BZR_HOME', '')
         self.bzrbranch = tempfile.mkdtemp()
-        branch = bzrlib.bzrdir.BzrDir.create_branch_convenience(
+        self.branch = bzrlib.bzrdir.BzrDir.create_branch_convenience(
             self.bzrbranch, force_new_tree=True)
-        tree = branch.bzrdir.open_workingtree()
+        self.tree = self.branch.bzrdir.open_workingtree()
+
+    config_template = """
+    [project]
+        [[branch]]
+            branch_name = 'branch'
+            folder = '%(branch)s'
+    """
+
+    def setUpLoggerhead(self):
+        ini = self.config_template%dict(branch=self.bzrbranch)
+
+        config = ConfigObj(ini.splitlines())
+        cherrypy.root = Root(config)
+
+    def tearDown(self):
+        if self.bzrbranch is not None:
+            shutil.rmtree(self.bzrbranch)
+        bzrlib.osutils.set_or_unset_env('BZR_HOME', self.old_bzrhome)
+
+class TestWithSimpleTree(BasicTests):
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.createBranch()
+
         f = open(os.path.join(self.bzrbranch, 'myfilename'), 'w')
-        self.filecontents = 'some\nmultiline\ndata'
+        self.filecontents = ('some\nmultiline\ndata\n'
+                             'with<htmlspecialchars\n')
         try:
             f.write(self.filecontents)
         finally:
             f.close()
-        tree.add('myfilename')
-        self.fileid = tree.path2id('myfilename')
-        self.msg = 'a very exciting commit message'
-        self.revid = tree.commit(message=self.msg)
-
-        ini = config_template%self.bzrbranch
-
-        config = ConfigObj(ini.splitlines())
-        cherrypy.root = Root(config)
-
-    def tearDown(self):
-        shutil.rmtree(self.bzrbranch)
-        bzrlib.osutils.set_or_unset_env('BZR_HOME', self.old_bzrhome)
-
-    # there are so i can run it with py.test and take advantage of the
-    # error reporting...
-    def setup_method(self, meth):
-        self.setUp()
-
-    def teardown_method(self, meth):
-        self.tearDown()
+        self.tree.add('myfilename')
+        self.fileid = self.tree.path2id('myfilename')
+        self.msg = 'a very exciting commit message <'
+        self.revid = self.tree.commit(message=self.msg)
+
+        self.setUpLoggerhead()
 
     def test_index(self):
         testutil.create_request('/')
@@ -68,15 +88,59 @@
 
     def test_changes(self):
         testutil.create_request('/project/branch/changes')
-        assert self.msg in cherrypy.response.body[0]
+        assert cgi.escape(self.msg) in cherrypy.response.body[0]
+
+    def test_changes_search(self):
+        testutil.create_request('/project/branch/changes?q=foo')
+        assert 'Sorry, no results found for your search.' in cherrypy.response.body[0]
 
     def test_annotate(self):
         testutil.create_request('/project/branch/annotate?'
                                 + 'file_id='+self.fileid)
         for line in self.filecontents.splitlines():
-            assert line in cherrypy.response.body[0]
+            assert cgi.escape(line) in cherrypy.response.body[0]
 
     def test_inventory(self):
         testutil.create_request('/project/branch/files')
         assert 'myfilename' in cherrypy.response.body[0]
 
+    def test_revision(self):
+        testutil.create_request('/project/branch/revision/' + self.revid)
+        assert 'myfilename' in cherrypy.response.body[0]
+
+class TestWithSimpleTreeAndCache(TestWithSimpleTree):
+    config_template = """
+    testing = True
+    [project]
+        [[branch]]
+            branch_name = 'branch'
+            folder = '%(branch)s'
+            cachepath = '%(branch)s/cache'
+    """
+
+class TestEmptyBranch(BasicTests):
+
+    def setUp(self):
+        BasicTests.setUp(self)
+        self.createBranch()
+        self.setUpLoggerhead()
+
+    def test_index(self):
+        testutil.create_request('/')
+        link = '<a href="/project/branch">branch</a>'
+        assert link in cherrypy.response.body[0]
+
+    def test_changes(self):
+        testutil.create_request('/project/branch/changes')
+        assert 'No revisions!' in cherrypy.response.body[0]
+
+class TestEmptyBranchWithCache(TestEmptyBranch):
+    config_template = """
+    testing = True
+    [project]
+        [[branch]]
+            branch_name = 'branch'
+            folder = '%(branch)s'
+            cachepath = '%(branch)s/cache'
+    """
+

=== modified file 'loggerhead/textindex.py'
--- loggerhead/textindex.py	2007-01-02 07:14:03 +0000
+++ loggerhead/textindex.py	2007-08-28 16:11:01 +0000
@@ -19,7 +19,7 @@
 """
 indexing of the comment text of revisions, for fast searching.
 
-two separate 'shelve' files are created:
+two separate database files are created:
 
     - recorded: revid -> 1 (if the revid is indexed)
     - index: 3-letter substring -> list(revids)
@@ -28,13 +28,13 @@
 import logging
 import os
 import re
-import shelve
 import threading
 import time
 
 from loggerhead import util
 from loggerhead.util import decorator
 from loggerhead.lockfile import LockFile
+from loggerhead.changecache import FakeShelf
 
 # if any substring index reaches this many revids, replace the entry with
 # an ALL marker -- it's not worth an explicit index.
@@ -69,21 +69,27 @@
         if not os.path.exists(cache_path):
             os.mkdir(cache_path)
         
-        self._recorded_filename = os.path.join(cache_path, 'textindex-recorded')
-        self._index_filename = os.path.join(cache_path, 'textindex')
+        self._recorded_filename = os.path.join(cache_path, 'textindex-recorded.sql')
+        self._index_filename = os.path.join(cache_path, 'textindex.sql')
         
         # use a lockfile since the cache folder could be shared across different processes.
         self._lock = LockFile(os.path.join(cache_path, 'index-lock'))
         self._closed = False
         
         self.log.info('Using search index; %d entries.', len(self))
+
+    def _index(self):
+        return FakeShelf(self._index_filename)
+
+    def _recorded(self):
+        return FakeShelf(self._recorded_filename)
     
     def _is_indexed(self, revid, recorded):
-        return recorded.get(util.to_utf8(revid), None) is not None
+        return recorded.get(util.to_utf8(revid)) is not None
         
     @with_lock
     def is_indexed(self, revid):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
+        recorded = self._recorded()
         try:
             return self._is_indexed(revid, recorded)
         finally:
@@ -91,9 +97,9 @@
     
     @with_lock
     def __len__(self):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
+        recorded = self._recorded()
         try:
-            return len(recorded)
+            return recorded.count()
         finally:
             recorded.close()
 
@@ -111,9 +117,11 @@
     
     @with_lock
     def full(self):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
+        recorded = self._recorded()
+        last_revid = util.to_utf8(self.history.last_revid)
         try:
-            return (len(recorded) >= len(self.history.get_revision_history())) and (util.to_utf8(self.history.last_revid) in recorded)
+            return (recorded.count() >= len(self.history.get_revision_history())
+                    and recorded.get(last_revid) is not None)
         finally:
             recorded.close()
 
@@ -126,7 +134,7 @@
             return
         for i in xrange(len(comment) - 2):
             sub = comment[i:i + 3]
-            revid_set = index.get(sub, None)
+            orig = revid_set = index.get(sub)
             if revid_set is None:
                 revid_set = set()
             elif revid_set == ALL:
@@ -135,26 +143,29 @@
             revid_set.add(change.revid)
             if len(revid_set) > ALL_THRESHOLD:
                 revid_set = ALL
-            index[sub] = revid_set
-        
-        recorded[util.to_utf8(change.revid)] = True
+            if orig is not None:
+                index.update([(sub, revid_set)], commit=False)
+            else:
+                index.add([(sub, revid_set)], commit=False)
+
+        recorded.add([(util.to_utf8(change.revid), True)], commit=False)
 
     @with_lock
     def index_changes(self, revid_list):
-        recorded = shelve.open(self._recorded_filename, 'c', protocol=2)
-        index = shelve.open(self._index_filename, 'c', protocol=2)
+        recorded = self._recorded()
+        index = self._index()
         try:
             revid_list = [r for r in revid_list if not self._is_indexed(r, recorded)]
             change_list = self.history.get_changes(revid_list)
             for change in change_list:
                 self._index_change(change, recorded, index)
         finally:
-            index.close()
-            recorded.close()
+            index.close(commit=True)
+            recorded.close(commit=True)
     
     @with_lock
     def find(self, text, revid_list=None):
-        index = shelve.open(self._index_filename, 'c', protocol=2)
+        index = self._index()
         try:
             text = normalize_string(text)
             if len(text) < 3:
@@ -167,7 +178,7 @@
             
             for i in xrange(len(text) - 2):
                 sub = text[i:i + 3]
-                revid_set = index.get(sub, None)
+                revid_set = index.get(sub)
                 if revid_set is None:
                     # zero matches, stop here.
                     return []

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2007-03-12 17:51:19 +0000
+++ loggerhead/util.py	2007-06-25 06:44:52 +0000
@@ -181,6 +181,7 @@
 # only do this if unicode turns out to be a problem
 #_BADCHARS_RE = re.compile(ur'[\u007f-\uffff]')
 
+# FIXME: get rid of this method; use fixed_width() and avoid XML().
 def html_clean(s):
     """
     clean up a string for html display.  expand any tabs, encode any html
@@ -188,11 +189,31 @@
     in displaying monospace text.
     """
     s = cgi.escape(s.expandtabs())
-#    s = _BADCHARS_RE.sub(lambda x: '&#%d;' % (ord(x.group(0)),), s)
     s = s.replace(' ', '&nbsp;')
     return s
 
 
+
+NONBREAKING_SPACE = u'\N{NO-BREAK SPACE}'
+
+def fixed_width(s):
+    """
+    expand tabs and turn spaces into "non-breaking spaces", so browsers won't
+    chop up the string.
+    """
+    if not isinstance(s, unicode):
+        # this kinda sucks.  file contents are just binary data, and no
+        # encoding metadata is stored, so we need to guess.  this is probably
+        # okay for most code, but for people using things like KOI-8, this
+        # will display gibberish.  we have no way of detecting the correct
+        # encoding to use.
+        try:
+            s = s.decode('utf-8')
+        except UnicodeDecodeError:
+            s = s.decode('iso-8859-15')
+    return s.expandtabs().replace(' ', NONBREAKING_SPACE)
+
+
 def fake_permissions(kind, executable):
     # fake up unix-style permissions given only a "kind" and executable bit
     if kind == 'directory':
@@ -272,13 +293,14 @@
     return out
     
 
-def fill_in_navigation(history, navigation):
+def fill_in_navigation(navigation):
     """
     given a navigation block (used by the template for the page header), fill
     in useful calculated values.
     """
-    navigation.position = history.get_revid_sequence(navigation.revid_list, navigation.revid)
-    if navigation.position is None:
+    if navigation.revid in navigation.revid_list: # XXX is this always true?
+        navigation.position = navigation.revid_list.index(navigation.revid)
+    else:
         navigation.position = 0
     navigation.count = len(navigation.revid_list)
     navigation.page_position = navigation.position // navigation.pagesize + 1

=== modified file 'start-loggerhead.py'
--- start-loggerhead.py	2007-06-18 23:07:53 +0000
+++ start-loggerhead.py	2007-06-19 00:09:43 +0000
@@ -22,7 +22,7 @@
     elif roll == 'weekly':
         h = logging.handlers.TimedRotatingFileHandler(filename, 'W0', 0, 100)
     else:
-        h = logging.handlers.FileHandler(filename)
+        h = logging.FileHandler(filename)
     return h
 
         
@@ -74,7 +74,7 @@
     if options.check:
         if daemon.is_running(pidfile):
             sys.exit(0)
-        sys.stderr.write('Did not find okulo running in %r; restarting...\n' % (pidfile,))
+        sys.stderr.write('Did not find loggerhead running in %r; restarting...\n' % (pidfile,))
     
     # read loggerhead config
     

